@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -301,7 +301,7 @@ const uint32 BlowFish::tempbf_S[4][256] =
 };
 
 void BlowFish::Blowfish_encipher(uint32 *xl,
-                                 uint32 *xr)
+                                 uint32 *xr) const
 {
   union aword Xl;
   union aword Xr;
@@ -325,7 +325,7 @@ void BlowFish::Blowfish_encipher(uint32 *xl,
 }
 
 void BlowFish::Blowfish_decipher(uint32 *xl,
-                                 uint32 *xr)
+                                 uint32 *xr) const
 {
   union aword Xl;
   union aword Xr;
@@ -424,7 +424,7 @@ BlowFish::~BlowFish()
   trashMemory(bf_S, sizeof(bf_S));
 }
 
-void BlowFish::Encrypt(const unsigned char *in, unsigned char *out)
+void BlowFish::Encrypt(const unsigned char *in, unsigned char *out) const
 {
   for (int x = 0; x < 8; x++)
     out[x] = in[x];
@@ -432,7 +432,7 @@ void BlowFish::Encrypt(const unsigned char *in, unsigned char *out)
     reinterpret_cast<uint32 *>(out + sizeof(uint32)));
 }
 
-void BlowFish::Decrypt(const unsigned char *in, unsigned char *out)
+void BlowFish::Decrypt(const unsigned char *in, unsigned char *out) const
 {
   for (int x = 0; x < 8; x++)
     out[x] = in[x];
@@ -441,7 +441,7 @@ void BlowFish::Decrypt(const unsigned char *in, unsigned char *out)
 }
 
 /*
-* Returns a BlowFish object set up for encryption or decrytion.
+* Returns a BlowFish object set up for encryption or decryption.
 *
 * The main issue here is that the BlowFish key is SHA1(passphrase|salt)
 * Aside from saving duplicate code, we win here by minimizing the exposure
@@ -453,8 +453,8 @@ void BlowFish::Decrypt(const unsigned char *in, unsigned char *out)
 * Note that it's the caller's responsibility to delete the BlowFish object allocated here
 */
 
-BlowFish *BlowFish::MakeBlowFish(const unsigned char *pass, int passlen,
-                                 const unsigned char *salt, int saltlen)
+BlowFish *BlowFish::MakeBlowFish(const unsigned char *pass, unsigned int passlen,
+                                 const unsigned char *salt, unsigned int saltlen)
 {
   unsigned char passkey[SHA1::HASHLEN];
   pws_os::mlock(passkey, sizeof(passkey));
