@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -404,11 +404,13 @@ struct XMLRecordWriter {
                                bforce_normal_entry, bXMLErrorsFound);
 
       if (bXMLErrorsFound) {
-        m_pRpt->WriteLine(_T("\t"), false);
-        m_pRpt->WriteLine(strXMLErrors.c_str());
+        if (m_pRpt != NULL) {
+          m_pRpt->WriteLine(_T("\t"), false);
+          m_pRpt->WriteLine(strXMLErrors.c_str());
+        }
         m_numXMLErrors++;
       } else
-        m_pRpt->WriteLine();
+        if (m_pRpt != NULL) m_pRpt->WriteLine();
 
       m_ofs.write(xml.c_str(),
                  static_cast<streamsize>(xml.length()));
@@ -580,14 +582,18 @@ int PWScore::WriteXMLFile(const StringX &filename,
     }
 
     ostringstreamT os;
-    os << "\t<EmptyGroups>" << endl;
-    for (size_t n = 0; n < vsubemptygroups.size(); n++) {
-      stringT sTemp = PWSUtil::GetSafeXMLString(vsubemptygroups[n]);
-      os << "\t\t<EGName>" << sTemp << "</EGName>" << endl;
+
+    // Don't write out XML empty groups if there aren't any!
+    if (!vsubemptygroups.empty()) {
+      os << "\t<EmptyGroups>" << endl;
+      for (size_t n = 0; n < vsubemptygroups.size(); n++) {
+        stringT sTemp = PWSUtil::GetSafeXMLString(vsubemptygroups[n]);
+        os << "\t\t<EGName>" << sTemp << "</EGName>" << endl;
+      }
+      os << "\t</EmptyGroups>" << endl << endl;
+      conv.ToUTF8(os.str().c_str(), utf8, utf8Len);
+      ofs.write(reinterpret_cast<const char *>(utf8), utf8Len);
     }
-    os << "\t</EmptyGroups>" << endl << endl;
-    conv.ToUTF8(os.str().c_str(), utf8, utf8Len);
-    ofs.write(reinterpret_cast<const char *>(utf8), utf8Len);
   }
 
   bool bStartComment(false);
@@ -916,25 +922,29 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
 #define HDR_MAP_ENTRY2(e, ie) {e, CItemData::ie},
     
         // These must be defined in the same order as Fields enum above, or it will assert below
-      
-        HDR_MAP_ENTRY(GROUPTITLE) HDR_MAP_ENTRY(USER)               HDR_MAP_ENTRY(PASSWORD)  HDR_MAP_ENTRY(URL)
-        HDR_MAP_ENTRY(AUTOTYPE)   HDR_MAP_ENTRY(CTIME)              HDR_MAP_ENTRY(PMTIME)    HDR_MAP_ENTRY(ATIME)
-        HDR_MAP_ENTRY(XTIME)      HDR_MAP_ENTRY(XTIME_INT)          HDR_MAP_ENTRY(RMTIME)    HDR_MAP_ENTRY(POLICY)
-        HDR_MAP_ENTRY(POLICYNAME) HDR_MAP_ENTRY2(HISTORY, PWHIST)   HDR_MAP_ENTRY(RUNCMD)    HDR_MAP_ENTRY(DCA)
-        HDR_MAP_ENTRY(SHIFTDCA)   HDR_MAP_ENTRY(EMAIL)              HDR_MAP_ENTRY(PROTECTED) HDR_MAP_ENTRY(SYMBOLS)
+        HDR_MAP_ENTRY(GROUPTITLE) HDR_MAP_ENTRY(USER)
+        HDR_MAP_ENTRY(PASSWORD)   HDR_MAP_ENTRY(URL)
+        HDR_MAP_ENTRY(AUTOTYPE)   HDR_MAP_ENTRY(CTIME)
+        HDR_MAP_ENTRY(PMTIME)     HDR_MAP_ENTRY(ATIME)
+        HDR_MAP_ENTRY(XTIME)      HDR_MAP_ENTRY(XTIME_INT)
+        HDR_MAP_ENTRY(RMTIME)     HDR_MAP_ENTRY(POLICY)
+        HDR_MAP_ENTRY(POLICYNAME) HDR_MAP_ENTRY2(HISTORY, PWHIST)
+        HDR_MAP_ENTRY(RUNCMD)     HDR_MAP_ENTRY(DCA)
+        HDR_MAP_ENTRY(SHIFTDCA)   HDR_MAP_ENTRY(EMAIL)
+        HDR_MAP_ENTRY(PROTECTED)  HDR_MAP_ENTRY(SYMBOLS)
         HDR_MAP_ENTRY(KBSHORTCUT) HDR_MAP_ENTRY(NOTES)
 
 #undef HDR_MAP_ENTRY
 #undef HDR_MAP_ENTRY2
     };
     
-    // make sure all elements are there
+  // make sure all elements are there
   static_assert((NumberOf(fieldMap) == NUMFIELDS), "Mismatch between fieldMap size and NUMFIELDS");
 
   to = 0;
   do {
     // make sure EXPORTHEADER has correct field names, though we have some resiliency below
-    ASSERT( vs_Header[itoken] == CItemData::FieldName(fieldMap[itoken].itemField));  
+    ASSERT( vs_Header[itoken] == CItemData::EngFieldName(fieldMap[itoken].itemField));
     from = s_header.find_first_not_of(pSeps, to);
     if (from == string::npos)
       break;
@@ -947,7 +957,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
       i_Offset[it - vs_Header.begin()] = itoken;
       num_found++;
     }
-    else if ( token == CItemData::FieldName(fieldMap[itoken].itemField) ) {
+    else if ( token == CItemData::EngFieldName(fieldMap[itoken].itemField) ) {
       // Column header might not match if it was exported from a version where EXPORTHEADER didn't
       // name the column titles correctly.  So compare directly with the corresponding field name.
       // Note that we are not searching here, unlike the above if block. The incorrect title has to
@@ -1364,9 +1374,6 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
       ci_temp.SetStatus(CItemData::ES_ADDED);
     }
 
-    // Get GUI to populate its field
-    GUISetupDisplayInfo(ci_temp);
-
     // Need to check that entry keyboard shortcut not already in use!
     int32 iKBShortcut;
     ci_temp.GetKBShortcut(iKBShortcut);
@@ -1391,6 +1398,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
         rpt.WriteLine(sxTemp.c_str());
         numshortcutsremoved++;
       }
+
       if (m_iAppHotKey == iKBShortcut) {
         // Remove it
         ci_temp.SetKBShortcut(0);
@@ -1807,7 +1815,6 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
 
     ci_temp.SetStatus(CItemData::ES_ADDED);
 
-    GUISetupDisplayInfo(ci_temp);
     Command *pcmd = AddEntryCommand::Create(this, ci_temp);
     pcmd->SetNoGUINotify();
     pmulticmds->Add(pcmd);
@@ -1990,7 +1997,7 @@ int PWScore::ImportKeePassV1CSVFile(const StringX &filename,
   for (size_t i = 0; i < hdr_tokens.size(); i++) {
     vector<StringX>::iterator it(std::find(vs_Header.begin(), vs_Header.end(), hdr_tokens[i]));
     if (it != vs_Header.end()) {
-      i_Offset[it - vs_Header.begin()] = i;
+      i_Offset[it - vs_Header.begin()] = (int)i;
       num_found++;
     }
   }
@@ -2289,9 +2296,6 @@ int PWScore::ImportKeePassV1CSVFile(const StringX &filename,
 
     ci_temp.SetStatus(CItemData::ES_ADDED);
 
-    // Get GUI to populate its field
-    GUISetupDisplayInfo(ci_temp);
-
     // Add to commands to execute
     Command *pcmd = AddEntryCommand::Create(this, ci_temp);
     pcmd->SetNoGUINotify();
