@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -25,6 +25,12 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
   const StringX::size_type val_len = stValue.length();
   const StringX::size_type obj_len = sx_Object.length();
 
+  StringX sx_Object_lc(sx_Object);
+  ToLower(sx_Object_lc);
+
+  StringX stValue_lc(stValue);
+  ToLower(stValue_lc);
+
   // Negative = Case   Sensitive
   // Positive = Case INsensitive
   switch (iFunction) {
@@ -40,56 +46,46 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
     case -MR_BEGINS:
     case  MR_BEGINS:
       if (obj_len >= val_len) {
-        sx_Object = sx_Object.substr(0, val_len);
-        return (((iFunction < 0) && (stValue == sx_Object)) ||
-                ((iFunction > 0) && CompareNoCase(stValue, sx_Object) == 0));
+        StringX sx_Subset = sx_Object.substr(0, val_len);
+        return (((iFunction < 0) && (stValue == sx_Subset)) ||
+                ((iFunction > 0) && CompareNoCase(stValue, sx_Subset) == 0));
       } else {
         return false;
       }
     case -MR_NOTBEGIN:
     case  MR_NOTBEGIN:
       if (obj_len >= val_len) {
-        sx_Object = sx_Object.substr(0, val_len);
-        return (((iFunction < 0) && (stValue != sx_Object)) ||
-                ((iFunction > 0) && CompareNoCase(stValue, sx_Object) != 0));
+        StringX sx_Subset = sx_Object.substr(0, val_len);
+        return (((iFunction < 0) && (stValue != sx_Subset)) ||
+                ((iFunction > 0) && CompareNoCase(stValue, sx_Subset) != 0));
       } else {
         return true;
       }
     case -MR_ENDS:
     case  MR_ENDS:
       if (obj_len > val_len) {
-        sx_Object = sx_Object.substr(obj_len - val_len);
-        return (((iFunction < 0) && (stValue == sx_Object)) ||
-                ((iFunction > 0) && CompareNoCase(stValue, sx_Object) == 0));
+        StringX sx_Subset = sx_Object.substr(obj_len - val_len);
+        return (((iFunction < 0) && (stValue == sx_Subset)) ||
+                ((iFunction > 0) && CompareNoCase(stValue, sx_Subset) == 0));
       } else {
         return false;
       }
     case -MR_NOTEND:
     case  MR_NOTEND:
       if (obj_len > val_len) {
-        sx_Object = sx_Object.substr(obj_len - val_len);
-        return (((iFunction < 0) && (stValue != sx_Object)) ||
-                ((iFunction > 0) && CompareNoCase(stValue, sx_Object) != 0));
+        StringX sx_Subset = sx_Object.substr(obj_len - val_len);
+        return (((iFunction < 0) && (stValue != sx_Subset)) ||
+                ((iFunction > 0) && CompareNoCase(stValue, sx_Subset) != 0));
       } else
         return true;
     case -MR_CONTAINS:
       return (sx_Object.find(stValue) != StringX::npos);
     case  MR_CONTAINS:
-    {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
-      return (sx_Object.find(stValue_lc) != StringX::npos);
-    }
+      return (sx_Object_lc.find(stValue_lc) != StringX::npos);
     case -MR_NOTCONTAIN:
       return (sx_Object.find(stValue) == StringX::npos);
     case  MR_NOTCONTAIN:
-    {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
-      return (sx_Object.find(stValue_lc) == StringX::npos);
-    }
+      return (sx_Object_lc.find(stValue_lc) == StringX::npos);
     case -MR_CNTNANY:
     {
       const charT *c = stValue.c_str();
@@ -102,12 +98,9 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
     }
     case  MR_CNTNANY:
     {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
       const charT *c = stValue_lc.c_str();
       for (size_t i = 0; i < stValue_lc.length(); i++) {
-        if (sx_Object.find(c, 0, 1) != StringX::npos)
+        if (sx_Object_lc.find(c, 0, 1) != StringX::npos)
           return true;
         c++;
       }
@@ -125,12 +118,9 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
     }
     case  MR_NOTCNTNANY:
     {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
       const charT *c = stValue_lc.c_str();
       for (size_t i = 0; i < stValue_lc.length(); i++) {
-        if (sx_Object.find(c, 0, 1) != StringX::npos)
+        if (sx_Object_lc.find(c, 0, 1) != StringX::npos)
           return false;
         c++;
       }
@@ -148,12 +138,9 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
     }
     case  MR_CNTNALL:
     {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
       const charT *c = stValue_lc.c_str();
       for (size_t i = 0; i < stValue_lc.length(); i++) {
-        if (sx_Object.find(c, 0, 1) == StringX::npos)
+        if (sx_Object_lc.find(c, 0, 1) == StringX::npos)
           return false;
         c++;
       }
@@ -171,12 +158,9 @@ bool PWSMatch::Match(const StringX &stValue, StringX sx_Object,
     }
     case  MR_NOTCNTNALL:
     {
-      ToLower(sx_Object);
-      StringX stValue_lc(stValue);
-      ToLower(stValue_lc);
       const charT *c = stValue_lc.c_str();
       for (size_t i = 0; i < stValue_lc.length(); i++) {
-        if (sx_Object.find(c, 0, 1) != StringX::npos)
+        if (sx_Object_lc.find(c, 0, 1) != StringX::npos)
           return false;
         c++;
       }
@@ -358,30 +342,34 @@ void PWSMatch::GetMatchType(MatchType mtype,
       LoadAString(cs2, fcase ? IDSC_CASE_SENSITIVE : IDSC_CASE_INSENSITIVE);
       break;
     case MT_INTEGER:
-      Format(cs1, _T("%d"), fnum1);
+      Format(cs1, L"%d", fnum1);
       if (bBetween)
-        Format(cs2, _T("%d"), fnum2);
+        Format(cs2, L"%d", fnum2);
       break;
     case MT_DATE:
     {
       if (fdatetype == 1 /* Relative */) {
-        Format(cs1, _T("%d"), fnum1);
+        Format(cs1, L"%d", fnum1);
         if (bBetween)
-          Format(cs2, _T("%d"), fnum2);
+          Format(cs2, L"%d", fnum2);
       } else {
         struct tm st_s;
         errno_t err;
         err = localtime_s(&st_s, &fdate1);
         ASSERT(err == 0);
-        TCHAR tc_buf1[80];
-        _tcsftime(tc_buf1, sizeof(tc_buf1) / sizeof(tc_buf1[0]), _T("%x"), &st_s);
-        cs1 = tc_buf1;
+        if (!err) {
+          TCHAR tc_buf1[80];
+          _tcsftime(tc_buf1, sizeof(tc_buf1) / sizeof(tc_buf1[0]), _T("%x"), &st_s);
+          cs1 = tc_buf1;
+        }
         if (bBetween) {
           err = localtime_s(&st_s, &fdate2);
           ASSERT(err == 0);
-          TCHAR tc_buf2[80];
-          _tcsftime(tc_buf2, sizeof(tc_buf2) / sizeof(tc_buf2[0]), _T("%x"), &st_s);
-          cs2 = tc_buf2;
+          if (!err) {
+            TCHAR tc_buf2[80];
+            _tcsftime(tc_buf2, sizeof(tc_buf2) / sizeof(tc_buf2[0]), _T("%x"), &st_s);
+            cs2 = tc_buf2;
+          }
         }
       }
       break;
@@ -389,6 +377,7 @@ void PWSMatch::GetMatchType(MatchType mtype,
     case MT_BOOL:
     case MT_PWHIST:
     case MT_POLICY:
+    case MT_ATTACHMENT:
       break;
     case MT_ENTRYTYPE:
       switch (etype) {
@@ -422,6 +411,30 @@ void PWSMatch::GetMatchType(MatchType mtype,
           id = IDSC_INVALID;
       }
       LoadAString(cs1, id);
+      if (fdca == -1) {
+        // Fill in the current message with the default action
+        short iDCA = (short)PWSprefs::GetInstance()->GetPref(mtype == MT_SHIFTDCA ?
+          PWSprefs::ShiftDoubleClickAction : PWSprefs::DoubleClickAction);
+        UINT id2;
+        switch (iDCA) {
+          case PWSprefs::DoubleClickCopyPassword:         id2 = IDSC_DCACOPYPASSWORD;    break;
+          case PWSprefs::DoubleClickViewEdit:             id2 = IDSC_DCAVIEWEDIT;        break;
+          case PWSprefs::DoubleClickAutoType:             id2 = IDSC_DCAAUTOTYPE;        break;
+          case PWSprefs::DoubleClickBrowse:               id2 = IDSC_DCABROWSE;          break;
+          case PWSprefs::DoubleClickCopyNotes:            id2 = IDSC_DCACOPYNOTES;       break;
+          case PWSprefs::DoubleClickCopyUsername:         id2 = IDSC_DCACOPYUSERNAME;    break;
+          case PWSprefs::DoubleClickCopyPasswordMinimize: id2 = IDSC_DCACOPYPASSWORDMIN; break;
+          case PWSprefs::DoubleClickBrowsePlus:           id2 = IDSC_DCABROWSEPLUS;      break;
+          case PWSprefs::DoubleClickRun:                  id2 = IDSC_DCARUN;             break;
+          case PWSprefs::DoubleClickSendEmail:            id2 = IDSC_DCASENDEMAIL;       break;
+          default:
+            ASSERT(0);
+            id2 = IDSC_INVALID;
+        }
+        stringT cs3;
+        LoadAString(cs3, id2);
+        Format(cs1, cs1.c_str(), cs3.c_str());
+      }
       break;
     case MT_ENTRYSTATUS:
       switch (estatus) {
@@ -436,11 +449,14 @@ void PWSMatch::GetMatchType(MatchType mtype,
       break;
     case MT_ENTRYSIZE:
       {
-        Format(cs1, _T("%d"), fnum1 >> (funit * 10));
+        Format(cs1, L"%d", fnum1 >> (funit * 10));
         if (bBetween)
-          Format(cs2, _T("%d"), fnum2 >> (funit * 10));
+          Format(cs2, L"%d", fnum2 >> (funit * 10));
       }
       break;
+    case MT_MEDIATYPE:
+      cs1 = fstring;
+      break;
     default:
       ASSERT(0);
   }
