@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -120,7 +120,7 @@ public:
   {m_nRecordsWithUnknownFields++;}
 
   // Clear out database structures and associated fields
-  void ClearData();
+  void ClearDBData();
   void ReInit(bool bNewfile = false);
 
   // Following used to read/write databases and Get/Set file name
@@ -145,7 +145,8 @@ public:
   int WriteExportFile(const StringX &filename, OrderedItemList *pOIL,
                       PWScore *pINcore, PWSfile::VERSION version,
                       std::vector<StringX> &vEmptyGroups, 
-                      bool bExportDBFilters, CReport *pRpt = NULL);
+                      bool bExportDBFilters,
+                      std::vector<pws_os::CUUID> &vuuidAddedBases, CReport *pRpt = NULL);
   int WriteV17File(const StringX &filename)
   {return WriteFile(filename, PWSfile::V17, false);}
   int WriteV2File(const StringX &filename)
@@ -244,13 +245,14 @@ public:
   bool IsLockedFile(const stringT &filename) const;
   void UnlockFile(const stringT &filename);
 
+  void SafeUnlockCurFile(); // unlocks current file iff we locked it.
+
   // Following 3 routines only for SaveAs to use a temporary lock handle
   // LockFile2, UnLockFile2 & MoveLock
   bool LockFile2(const stringT &filename, stringT &locker);
   void UnlockFile2(const stringT &filename);
   void MoveLock()
-  {m_lockFileHandle = m_lockFileHandle2; m_lockFileHandle2 = INVALID_HANDLE_VALUE;
-   m_LockCount = m_LockCount2; m_LockCount2 = 0;}
+  {m_lockFileHandle = m_lockFileHandle2; m_lockFileHandle2 = INVALID_HANDLE_VALUE;}
 
   // Set application data
   void SetApplicationNameAndVersion(const stringT &appName, DWORD dwMajorMinor,
@@ -311,9 +313,11 @@ public:
   int Execute(Command *pcmd);
   void Undo();
   void Redo();
-  void ClearCommands();
+  void ClearCommands();  // This should be private to prevent UI calling directly but called by coretest
   bool AnyToUndo() const;
   bool AnyToRedo() const;
+  Command * GetRedoCommand();
+  Command * GetUndoCommand();
 
   // Find in m_pwlist by group, title and user name, exact match
   ItemListIter Find(const StringX &a_group,
@@ -338,6 +342,7 @@ public:
 
   // alias/base and shortcut/base handling
   void SortDependents(UUIDVector &dlist, StringX &csDependents);
+  void SortDependents(UUIDVector &dlist, std::vector<StringX> &vsxDependents);
   size_t NumAliases(const pws_os::CUUID &base_uuid) const
   {return m_base2aliases_mmap.count(base_uuid);}
   size_t NumShortcuts(const pws_os::CUUID &base_uuid) const
@@ -378,8 +383,7 @@ public:
   bool GetDBNotificationState()
   {return m_bNotifyDB;}
 
-  void GUISetupDisplayInfo(CItemData &ci);
-  void GUIRefreshEntry(const CItemData &ci);
+  void GUIRefreshEntry(const CItemData &ci, bool bAllowFail = false);
   void UpdateWizard(const stringT &s);
 
   // Get/Set Display information from/to database
@@ -479,8 +483,9 @@ private:
                                       SavePWHistoryMap &mapSavedHistory);
   virtual void UndoUpdatePasswordHistory(SavePWHistoryMap &mapSavedHistory);
 
-  virtual int DoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
-  virtual void UndoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
+  virtual int DoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath,
+                            MultiCommands * &pmulticmds);
+  virtual void UndoRenameGroup(MultiCommands *pmulticmds);
 
   virtual int DoChangeHeader(const StringX &sxNewValue, const PWSfile::HeaderType ht);
   virtual void UndoChangeHeader(const StringX &sxOldValue, const PWSfile::HeaderType ht);
@@ -550,8 +555,6 @@ private:
 
   HANDLE m_lockFileHandle;
   HANDLE m_lockFileHandle2;
-  int m_LockCount;
-  int m_LockCount2;
 
   stringT m_AppNameAndVersion;
   PWSfile::VERSION m_ReadFileVersion;
@@ -585,10 +588,6 @@ private:
   const ItemMMap &GetBase2ShortcutsMmap() const {return m_base2shortcuts_mmap;}
   void SetBase2ShortcutsMmap(ItemMMap &b2smm) {m_base2shortcuts_mmap = b2smm;}
   
-  // Changed groups
-  std::vector<StringX> m_vNodes_Modified;
-  std::vector<StringX> m_saved_vNodes_Modified;
-
   // Following are private in PWScore, public in CommandInterface:
   void AddChangedNodes(StringX path);
 
@@ -606,9 +605,12 @@ private:
   std::bitset<UIInterFace::NUM_SUPPORTED> m_bsSupportedFunctions;
   
   void NotifyGUINeedsUpdating(UpdateGUICommand::GUI_Action, const pws_os::CUUID &,
-                              CItemData::FieldType ft = CItemData::START,
-                              bool bUpdateGUI = true);
+                              CItemData::FieldType ft = CItemData::START);
 
+  // Version for groups
+  void NotifyGUINeedsUpdating(UpdateGUICommand::GUI_Action ga,
+                              const std::vector<StringX> &vGroups);
+  
   // Create header for included(Text) and excluded(XML) exports
   StringX BuildHeader(const CItemData::FieldBits &bsFields, const bool bIncluded);
 
