@@ -15,12 +15,14 @@
 #include "core.h"
 #include "StringXStream.h"
 #include "PWPolicy.h"
+#include "./UTF8Conv.h"
 
 #include "Util.h"
 
 #include "os/debug.h"
 #include "os/pws_tchar.h"
 #include "os/dir.h"
+#include "os/file.h"
 
 #include <stdio.h>
 #ifdef _WIN32
@@ -859,3 +861,52 @@ bool PWSUtil::pull_time(time_t &t, const unsigned char *data, size_t len)
   }
   return true;
 }
+
+bool FindNoCase( const StringX& src, const StringX& dest)
+{
+    StringX srcLower = src;
+    ToLower(srcLower);
+
+    StringX destLower = dest;
+    ToLower(destLower);
+
+    return destLower.find(srcLower) != StringX::npos;
+}
+
+std::string toutf8(const std::wstring &w)
+{
+  CUTF8Conv conv;
+  const unsigned char *utf8str = nullptr;
+  size_t length = 0;
+  if (conv.ToUTF8(std2stringx(w), utf8str, length))
+    return string{ reinterpret_cast<const char *>(utf8str), length};
+  return string{};
+}
+
+bool PWSUtil::loadFile(const StringX &filename, StringXStream &stream) {
+  // We need to use FOpen as the file name/file path may contain non-Latin
+  // characters
+  FILE *fs = pws_os::FOpen(filename.c_str(), _T("rb"));
+  if (fs == nullptr)
+    return false;
+
+  // when using wifstream, each byte will be converted to wchar_t, but we need to load
+  // each n-bytes as one wchar, so read in whole file and put it in a StringXStream
+  const size_t BUFFER_SIZE = 1024;
+
+  wchar_t buffer[BUFFER_SIZE];
+  bool bError(false);
+  while(!feof(fs)) {
+    int count = fread(buffer, sizeof(wchar_t), BUFFER_SIZE, fs);
+    if (ferror(fs)) {
+      bError = true;
+      break;
+    }
+    stream.write(buffer, count);
+  }
+
+  // Close the file
+  fclose(fs);
+
+  return !bError;
+}
