@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -87,6 +87,11 @@ BOOL CAboutDlg::OnInitDialog()
                         static_cast<LPCWSTR>(Revision));
   }
 
+#if _WIN64
+  // Only add platform information for 64-bit build
+  m_appversion += L" 64-bit";
+#endif
+
 #ifdef _DEBUG
   m_appversion += L" [D]";
 #endif
@@ -109,7 +114,7 @@ BOOL CAboutDlg::OnInitDialog()
     GetDlgItem(IDC_TAKETESTDUMP)->EnableWindow();
   }
 
-  return TRUE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 bool CAboutDlg::OnCheckVersion(const CString &URL, const CString & /* lpszFName */, LPARAM instance)
@@ -181,6 +186,7 @@ void CAboutDlg::CheckNewVer()
   m_RECExNewVerStatus.SetWindowText(m_newVerStatus);
   m_RECExNewVerStatus.Invalidate();
   UpdateData(FALSE);
+
   GetDlgItem(IDOK)->SetFocus();
 }
 
