@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -26,6 +26,8 @@
 
 #include "os/file.h"
 
+#include "os/debug.h"
+
 #include <shlwapi.h>
 #include <fstream>
 #include <limits>
@@ -42,10 +44,12 @@ static char THIS_FILE[] = __FILE__;
 
 static wchar_t PSSWDCHAR = L'*';
 
-CSecString CAddEdit_Basic::HIDDEN_NOTES;
+HANDLE CAddEdit_Basic::ghEvents[2];
 
 CString CAddEdit_Basic::CS_SHOW;
 CString CAddEdit_Basic::CS_HIDE;
+CString CAddEdit_Basic::CS_EXTERNAL_EDITOR;
+CString CAddEdit_Basic::CS_HIDDEN_NOTES;
 
 /////////////////////////////////////////////////////////////////////////////
 // CAddEdit_Basic property page
@@ -56,21 +60,36 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
   : CAddEdit_PropertyPage(pParent,
     CAddEdit_Basic::IDD, CAddEdit_Basic::IDD_SHORT,
     pAEMD),
-  m_bInitdone(false), m_thread(NULL), m_isNotesHidden(false), m_NotesFirstVisibleLine(-1)
+  m_bInitdone(false), m_thread(NULL), m_isNotesHidden(false),
+  m_bUsingNotesExternalEditor(false)
 {
   if (CS_SHOW.IsEmpty()) { // one-time initializations
-    HIDDEN_NOTES.LoadString(IDS_HIDDENNOTES);
     CS_SHOW.LoadString(IDS_SHOWPASSWORDTXT);
     CS_HIDE.LoadString(IDS_HIDEPASSWORDTXT);
+    CS_HIDDEN_NOTES.LoadString(IDS_HIDDENNOTES);
+    CS_EXTERNAL_EDITOR.LoadString(IDS_NOTES_IN_EXTERNAL_EDITOR);
   }
 
+  // Clear external editor events
+  ghEvents[0] = ghEvents[1] = NULL;
+
   PWSprefs *prefs = PWSprefs::GetInstance();
 
   // Setup
   m_bWordWrap = prefs->GetPref(PWSprefs::NotesWordWrap);
 
   m_password = m_password2 = M_realpassword();
-  m_notes = M_realnotes().Left(MAXTEXTCHARS);
+
+  if ((!M_pcore()->IsReadOnly() && M_protected() == 0) && 
+      M_notes().GetLength() > MAXTEXTCHARS) {
+    // Only truncate if in Edit mode
+    M_notes() = M_notes().Left(MAXTEXTCHARS);
+
+    CGeneralMsgBox gmb;
+    CString cs_text, cs_title(MAKEINTRESOURCE(IDS_WARNINGTEXTLENGTH));
+    cs_text.Format(IDS_TRUNCATETEXT, MAXTEXTCHARS);
+    gmb.MessageBox(cs_text, cs_title, MB_OK | MB_ICONEXCLAMATION);
+  }
 
   // Set up right-click Notes context menu additions
   std::vector<st_context_menu> vmenu_items;
@@ -110,7 +129,7 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
   m_ex_notes.SetContextMenu(vmenu_items);
 }
 
-void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
+void CAddEdit_Basic::DoDataExchange(CDataExchange *pDX)
 {
   CAddEdit_PropertyPage::DoDataExchange(pDX);
 
@@ -118,13 +137,12 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
   m_ex_password.DoDDX(pDX, m_password);
   m_ex_password2.DoDDX(pDX, m_password2);
 
-  DDX_Text(pDX, IDC_NOTES, (CString&)m_notes);
-
   DDX_CBString(pDX, IDC_GROUP, (CString&)M_group());
   DDX_Text(pDX, IDC_TITLE, (CString&)M_title());
   DDX_Text(pDX, IDC_USERNAME, (CString&)M_username());
   DDX_Text(pDX, IDC_URL, (CString&)M_URL());
   DDX_Text(pDX, IDC_EMAIL, (CString&)M_email());
+  DDX_Text(pDX, IDC_NOTES, (CString&)M_notes());
 
   DDX_Control(pDX, IDC_GROUP, m_ex_group);
   DDX_Control(pDX, IDC_TITLE, m_ex_title);
@@ -132,10 +150,11 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
   DDX_Control(pDX, IDC_PASSWORD, m_ex_password);
   DDX_Control(pDX, IDC_PASSWORD2, m_ex_password2);
   DDX_Control(pDX, IDC_NOTES, m_ex_notes);
+  DDX_Control(pDX, IDC_HIDDEN_NOTES, m_ex_hidden_notes);
   DDX_Control(pDX, IDC_URL, m_ex_URL);
   DDX_Control(pDX, IDC_EMAIL, m_ex_email);
   DDX_Control(pDX, IDC_MYBASE, m_ex_base);
-  DDX_Control(pDX, IDC_VIEWDEPENDENTS, m_ViewDependentsBtn);
+  DDX_Control(pDX, IDC_LISTDEPENDENTS, m_cmbDependents);
 
   DDX_Control(pDX, IDC_STATIC_GROUP, m_stc_group);
   DDX_Control(pDX, IDC_STATIC_TITLE, m_stc_title);
@@ -144,6 +163,8 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
   DDX_Control(pDX, IDC_STATIC_NOTES, m_stc_notes);
   DDX_Control(pDX, IDC_STATIC_URL, m_stc_URL);
   DDX_Control(pDX, IDC_STATIC_EMAIL, m_stc_email);
+  DDX_Control(pDX, IDC_STATIC_ISANALIAS, m_stc_isdependent);
+  DDX_Control(pDX, IDC_STATIC_DEPENDENT, m_stc_dependent);
 
   DDX_Control(pDX, IDC_SMARTLABELHELP, m_Help1);
   DDX_Control(pDX, IDC_PASSWORDHELP, m_Help2);
@@ -162,7 +183,6 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
   ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
   ON_BN_CLICKED(IDC_LAUNCH, OnLaunch)
   ON_BN_CLICKED(IDC_SENDEMAIL, OnSendEmail)
-  ON_BN_CLICKED(IDC_VIEWDEPENDENTS, OnViewDependents)
 
   ON_CBN_SELCHANGE(IDC_GROUP, OnGroupComboChanged)
   ON_CBN_EDITCHANGE(IDC_GROUP, OnGroupComboChanged)
@@ -177,7 +197,6 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
 
   ON_EN_SETFOCUS(IDC_PASSWORD, OnENSetFocusPassword)
   ON_EN_SETFOCUS(IDC_PASSWORD2, OnENSetFocusPassword2)
-  ON_EN_SETFOCUS(IDC_NOTES, OnENSetFocusNotes)
   ON_EN_KILLFOCUS(IDC_NOTES, OnENKillFocusNotes)
 
   ON_CONTROL_RANGE(STN_CLICKED, IDC_STATIC_GROUP, IDC_STATIC_EMAIL, OnSTCExClicked)
@@ -190,6 +209,7 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
 
   // Common
   ON_MESSAGE(PSM_QUERYSIBLINGS, OnQuerySiblings)
+  ON_NOTIFY(PSN_KILLACTIVE, 0, OnPageKillActive)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -203,8 +223,8 @@ BOOL CAddEdit_Basic::OnInitDialog()
   ModifyStyleEx(0, WS_EX_CONTROLPARENT);
 
   Fonts *pFonts = Fonts::GetInstance();
-  pFonts->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD));
-  pFonts->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD2));
+  pFonts->ApplyPasswordFont(&m_ex_password);
+  pFonts->ApplyPasswordFont(&m_ex_password2);
 
   // Get Add/Edit font
   CFont *pFont = pFonts->GetAddEditFont();
@@ -227,9 +247,11 @@ BOOL CAddEdit_Basic::OnInitDialog()
 
   // Set Notes font!
   if (prefs->GetPref(PWSprefs::NotesFont).empty()) {
-    m_ex_notes.SetFont(pFonts->GetCurrentFont());
+    m_ex_notes.SetFont(pFonts->GetAddEditFont());
+    m_ex_hidden_notes.SetFont(pFonts->GetAddEditFont());
   } else {
     m_ex_notes.SetFont(pFonts->GetNotesFont());
+    m_ex_hidden_notes.SetFont(pFonts->GetNotesFont());
   }
 
   if (InitToolTip(TTS_BALLOON | TTS_NOPREFIX, 0)) {
@@ -290,16 +312,17 @@ BOOL CAddEdit_Basic::OnInitDialog()
     CancelToClose();
 
     // Disable Group Combo
-    GetDlgItem(IDC_GROUP)->EnableWindow(FALSE);
+    m_ex_group.EnableWindow(FALSE);
 
     // Disable normal Edit controls
-    GetDlgItem(IDC_TITLE)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_USERNAME)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_PASSWORD)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_PASSWORD2)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_NOTES)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_URL)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_EMAIL)->SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_title.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_username.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_password.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_password2.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_notes.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_hidden_notes.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_URL.SendMessage(EM_SETREADONLY, TRUE, 0);
+    m_ex_email.SendMessage(EM_SETREADONLY, TRUE, 0);
 
     // Disable Button
     GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
@@ -340,26 +363,28 @@ BOOL CAddEdit_Basic::OnInitDialog()
   CString cs_text;
   if (M_original_entrytype() == CItemData::ET_ALIASBASE ||
       M_original_entrytype() == CItemData::ET_SHORTCUTBASE) {
-    // Show button to allow users to view dependents
-    cs_text.LoadString(M_original_entrytype() == CItemData::ET_ALIASBASE ?
-                       IDS_VIEWALIASESBTN : IDS_VIEWSHORTCUTSBTN);
-    GetDlgItem(IDC_VIEWDEPENDENTS)->SetWindowText(cs_text);
-    GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_SHOW);
+    // Show ComboBox to allow users to view dependents
+    SetUpDependentsCombo();
+    m_cmbDependents.ShowWindow(SW_SHOW);
+    m_cmbDependents.SetCurSel(0);
 
     cs_text.LoadString(M_original_entrytype() == CItemData::ET_ALIASBASE ?
                        IDS_ISANALIASBASE : IDS_ISASHORTCUTBASE);
-    GetDlgItem(IDC_STATIC_ISANALIAS)->SetWindowText(cs_text);
-    GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_SHOW);
+    m_stc_isdependent.SetWindowText(cs_text);
+
+    m_stc_isdependent.ShowWindow(SW_SHOW);
     m_ex_base.ShowWindow(SW_HIDE);
+    m_stc_dependent.ShowWindow(SW_SHOW);
   } else if (M_original_entrytype() == CItemData::ET_ALIAS) {
     // Update password to alias form
     // Show text stating that it is an alias
     m_password = m_password2 = M_base();
-    GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
+    m_cmbDependents.ShowWindow(SW_HIDE);
 
-    GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_SHOW);
+    m_stc_isdependent.ShowWindow(SW_SHOW);
     m_ex_base.SetWindowText(M_base());
     m_ex_base.ShowWindow(SW_SHOW);
+    m_stc_dependent.ShowWindow(SW_HIDE);
 
     // Swap Help buttons
     m_Help2.EnableWindow(FALSE);
@@ -369,9 +394,10 @@ BOOL CAddEdit_Basic::OnInitDialog()
     m_Help3.ShowWindow(SW_SHOW);
   } else if (M_original_entrytype() == CItemData::ET_NORMAL) {
     // Normal - do none of the above
-    GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_HIDE);
+    m_cmbDependents.ShowWindow(SW_HIDE);
+    m_stc_isdependent.ShowWindow(SW_HIDE);
     m_ex_base.ShowWindow(SW_HIDE);
+    m_stc_dependent.ShowWindow(SW_HIDE);
   }
 
   if (prefs->GetPref(PWSprefs::ShowPWDefault)) {
@@ -380,25 +406,25 @@ BOOL CAddEdit_Basic::OnInitDialog()
     HidePassword();
   }
 
+  // Set "Hidden Notes" text
+  m_ex_hidden_notes.SetWindowText(CS_HIDDEN_NOTES);
+  // Make read only
+  m_ex_hidden_notes.SendMessage(EM_SETREADONLY, TRUE, 0);
+
   if (prefs->GetPref(PWSprefs::ShowNotesDefault)) {
-    ShowNotes();
+    ShowNotes(true);
   } else {
-    HideNotes();
-    m_ex_notes.EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
-    m_ex_notes.EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
+    HideNotes(true);
   }
 
-  CHARFORMAT cf = {0};
+  // Get current font size
+  CHARFORMAT2 cf;
+  memset(&cf, 0, sizeof(cf));
   cf.cbSize = sizeof(cf);
-
-  m_ex_notes.SetSel(0, -1);
-  m_ex_notes.SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
+  m_ex_notes.GetDefaultCharFormat(cf);
+  ASSERT((cf.dwMask & CFM_SIZE) == CFM_SIZE);
   m_iPointSize = cf.yHeight / 20;
 
-  // Set initial Word Wrap
-  m_ex_notes.SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
-  m_ex_notes.UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
-
   // Load copy password bitmap
   UINT nImageID = PWSprefs::GetInstance()->GetPref(PWSprefs::UseNewToolbar) ?
     IDB_COPYPASSWORD_NEW : IDB_COPYPASSWORD_CLASSIC;
@@ -417,9 +443,13 @@ BOOL CAddEdit_Basic::OnInitDialog()
       pBtn->SetBitmap(m_CopyPswdBitmap);
   }
 
+  // Set initial Word Wrap
+  m_ex_notes.SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
+  m_ex_notes.UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
+
   UpdateData(FALSE);
   m_bInitdone = true;
-  return TRUE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 void CAddEdit_Basic::OnHelp()
@@ -494,6 +524,13 @@ BOOL CAddEdit_Basic::OnKillActive()
   return CAddEdit_PropertyPage::OnKillActive();
 }
 
+void CAddEdit_Basic::OnPageKillActive(NMHDR *, LRESULT *pLResult)
+{
+  // Don't allow page switching if Notes being edited in the user's
+  // external editor
+  *pLResult = m_bUsingNotesExternalEditor ? 1 : 0;
+}
+
 LRESULT CAddEdit_Basic::OnQuerySiblings(WPARAM wParam, LPARAM )
 {
   UpdateData(TRUE);
@@ -506,7 +543,7 @@ LRESULT CAddEdit_Basic::OnQuerySiblings(WPARAM wParam, LPARAM )
           if (M_group()        != M_pci()->GetGroup()      ||
               M_title()        != M_pci()->GetTitle()      ||
               M_username()     != M_pci()->GetUser()       ||
-              M_realnotes()    != M_originalrealnotesTRC() ||
+              M_notes()        != M_originalnotesTRC()     ||
               M_URL()          != M_pci()->GetURL()        ||
               M_email()        != M_pci()->GetEmail()      ||
               (M_ipolicy()     != NAMED_POLICY &&
@@ -519,7 +556,7 @@ LRESULT CAddEdit_Basic::OnQuerySiblings(WPARAM wParam, LPARAM )
               !M_title().IsEmpty()        ||
               !M_username().IsEmpty()     ||
               !M_realpassword().IsEmpty() ||
-              !M_realnotes().IsEmpty()    ||
+              !M_notes().IsEmpty()        ||
               !M_URL().IsEmpty()          ||
               !M_email().IsEmpty()        ||
               !M_symbols().IsEmpty()        )
@@ -537,7 +574,7 @@ LRESULT CAddEdit_Basic::OnQuerySiblings(WPARAM wParam, LPARAM )
   return 0L;
 }
 
-BOOL CAddEdit_Basic::PreTranslateMessage(MSG* pMsg)
+BOOL CAddEdit_Basic::PreTranslateMessage(MSG *pMsg)
 {
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_F1) {
     PostMessage(WM_COMMAND, MAKELONG(ID_HELP, BN_CLICKED), NULL);
@@ -546,10 +583,23 @@ BOOL CAddEdit_Basic::PreTranslateMessage(MSG* pMsg)
 
   RelayToolTipEvent(pMsg);
 
+  // Handle if user clicks on Hidden Notes
+  if (m_ex_hidden_notes.m_hWnd == pMsg->hwnd) {
+    switch (pMsg->message) {
+    case WM_LBUTTONDOWN:
+    case WM_RBUTTONDOWN:
+      if (!m_bUsingNotesExternalEditor)
+        ShowNotes();
+    case WM_LBUTTONUP:
+    case WM_RBUTTONUP:
+      return TRUE;
+    }
+  }
+
   // Ctrl + 'key' in Notes
   if (pMsg->message == WM_KEYDOWN &&
-    (GetKeyState(VK_CONTROL) & 0x8000) == 0x8000 &&
-    m_ex_notes.m_hWnd == ::GetFocus()) {
+      (GetKeyState(VK_CONTROL) & 0x8000) == 0x8000 &&
+      m_ex_notes.m_hWnd == ::GetFocus()) {
     switch (pMsg->wParam) {
     case 'A':
       // Ctrl+A (Select All), then SelectAllNotes
@@ -568,7 +618,7 @@ BOOL CAddEdit_Basic::PreTranslateMessage(MSG* pMsg)
   }
 
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_CONTROL &&
-    GetDlgItem(IDC_LAUNCH)->IsWindowEnabled()) {
+      GetDlgItem(IDC_LAUNCH)->IsWindowEnabled()) {
     CString cs_text(MAKEINTRESOURCE(m_bLaunchPlus ? IDS_LAUNCH : IDS_LAUNCHPLUS));
     GetDlgItem(IDC_LAUNCH)->SetWindowText(cs_text);
     m_bLaunchPlus = !m_bLaunchPlus;
@@ -598,7 +648,7 @@ BOOL CAddEdit_Basic::OnApply()
   M_email().EmptyIfOnlyWhiteSpace();
   M_symbols().EmptyIfOnlyWhiteSpace();
 
-  m_notes.EmptyIfOnlyWhiteSpace();
+  M_notes().EmptyIfOnlyWhiteSpace();
 
   if (m_password.IsOnlyWhiteSpace()) {
     m_password.Empty();
@@ -609,12 +659,9 @@ BOOL CAddEdit_Basic::OnApply()
   if (!m_isPWHidden || m_password != HIDDEN_PASSWORD)
     M_realpassword() = m_password;
 
-  if (!m_isNotesHidden)
-    M_realnotes() = m_notes;
-
   UpdateData(FALSE);
 
-  //Check that data is valid
+  // Check that data is valid
   if (M_title().IsEmpty()) {
     gmb.AfxMessageBox(IDS_MUSTHAVETITLE);
     pFocus = &m_ex_title;
@@ -642,39 +689,15 @@ BOOL CAddEdit_Basic::OnApply()
 
   // If there is a matching entry in our list, tell the user to try again.
   listindex = GetMainDlg()->Find(M_group(), M_title(), M_username());
-  if (M_uicaller() == IDS_ADDENTRY) {
-    // Add entry
-    if (listindex != GetMainDlg()->End()) {
-      CSecString temp;
-      if (M_group().IsEmpty())
-        if (M_username().IsEmpty())
-          temp.Format(IDS_ENTRYEXISTS3, static_cast<LPCWSTR>(M_title()));
-        else
-          temp.Format(IDS_ENTRYEXISTS2, static_cast<LPCWSTR>(M_title()),
-                      static_cast<LPCWSTR>(M_username()));
-      else
-        if (M_username().IsEmpty())
-          temp.Format(IDS_ENTRYEXISTS1, static_cast<LPCWSTR>(M_group()),
-                      static_cast<LPCWSTR>(M_title()));
-        else
-          temp.Format(IDS_ENTRYEXISTS, static_cast<LPCWSTR>(M_group()),
-                      static_cast<LPCWSTR>(M_title()),
-                      static_cast<LPCWSTR>(M_username()));
-
-      gmb.AfxMessageBox(temp);
+  if (listindex != GetMainDlg()->End()) {
+    if (M_uicaller() == IDS_ADDENTRY) { // Add entry
+      gmb.AfxMessageBox(IDS_ENTRYEXISTS, MB_OK | MB_ICONASTERISK);
       pFocus = &m_ex_title;
       goto error;
-    }
-  } else {
-    // Edit entry
-    if (listindex != GetMainDlg()->End()) {
+    } else { // Edit entry
       const CItemData &listItem = GetMainDlg()->GetEntryAt(listindex);
       if (listItem.GetUUID() != M_pci()->GetUUID()) {
-        CSecString temp;
-        temp.Format(IDS_ENTRYEXISTS, static_cast<LPCWSTR>(M_group()),
-                    static_cast<LPCWSTR>(M_title()),
-                    static_cast<LPCWSTR>(M_username()));
-        gmb.AfxMessageBox(temp);
+        gmb.AfxMessageBox(IDS_ENTRYEXISTS, MB_OK | MB_ICONASTERISK);
         pFocus = &m_ex_title;
         goto error;
       }
@@ -715,7 +738,7 @@ BOOL CAddEdit_Basic::OnApply()
       cs_errmsg.Format(M_original_entrytype() == CItemData::ET_ALIASBASE ?
                        IDS_CHANGINGBASEENTRY1 : IDS_CHANGINGBASEENTRY2,
                        static_cast<LPCWSTR>(cs_alias));
-      int rc = gmb.MessageBox(cs_errmsg, cs_title, MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2);
+      int rc = (int)gmb.MessageBox(cs_errmsg, cs_title, MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2);
 
       if (rc == IDNO) {
         UpdateData(FALSE);
@@ -806,11 +829,12 @@ void CAddEdit_Basic::ShowHideBaseInfo(const CItemData::EntryType &entrytype, CSe
 {
   switch (entrytype) {
   case CItemData::ET_ALIAS:
-    GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_SHOW);
+    m_cmbDependents.ShowWindow(SW_HIDE);
+    m_stc_isdependent.ShowWindow(SW_SHOW);
 
     m_ex_base.SetWindowText(csBase);
     m_ex_base.ShowWindow(SW_SHOW);
+    m_stc_dependent.ShowWindow(SW_HIDE);
 
     // Swap help buttons/information
     m_Help2.EnableWindow(FALSE);
@@ -819,11 +843,12 @@ void CAddEdit_Basic::ShowHideBaseInfo(const CItemData::EntryType &entrytype, CSe
     m_Help3.ShowWindow(SW_SHOW);
     break;
   case CItemData::ET_NORMAL:
-    GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_HIDE);
+    m_cmbDependents.ShowWindow(SW_HIDE);
+    m_stc_isdependent.ShowWindow(SW_HIDE);
 
     m_ex_base.SetWindowText(L"");
     m_ex_base.ShowWindow(SW_HIDE);
+    m_stc_dependent.ShowWindow(SW_HIDE);
 
     // Swap help buttons/information
     m_Help2.EnableWindow(TRUE);
@@ -936,48 +961,52 @@ LRESULT CAddEdit_Basic::OnZoomNotes(WPARAM, LPARAM lParam)
   if ((lParam < 0 && m_iPointSize <= 6) || (lParam > 0 && m_iPointSize >= 72))
     return 0L;
 
-  WPARAM wp_increment = (lParam > 0 ? 1 : -1) * 2;
+  UINT wp_increment = (lParam > 0 ? 1 : -1) * 2;
 
-  long nStart, nEnd;
-  m_ex_notes.GetSel(nStart, nEnd);
-
-  m_ex_notes.SetSel(0, -1);
-  m_ex_notes.HideSelection(TRUE, FALSE);
-  m_ex_notes.SendMessage(EM_SETFONTSIZE, wp_increment, 0);
-
-  CHARFORMAT cf = {0};
+  CHARFORMAT2 cf;
+  memset(&cf, 0, sizeof(cf));
   cf.cbSize = sizeof(cf);
+  cf.dwMask = CFM_SIZE;
+  cf.yHeight = (m_iPointSize + wp_increment) * 20;
+  m_ex_notes.SetDefaultCharFormat(cf);
 
-  m_ex_notes.SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
+  memset(&cf, 0, sizeof(cf));
+  cf.cbSize = sizeof(cf);
+  m_ex_notes.GetDefaultCharFormat(cf);
+  ASSERT((cf.dwMask & CFM_SIZE) == CFM_SIZE);
   m_iPointSize = cf.yHeight / 20;
 
-  m_ex_notes.SetSel(nStart, nEnd);
-
   SetZoomMenu();
   return 0L;
 }
 
-void CAddEdit_Basic::ShowNotes()
+void CAddEdit_Basic::ShowNotes(const bool bForceShow)
 {
-  if (m_isNotesHidden) { // idempotent
+  if (bForceShow || m_isNotesHidden) {
     m_isNotesHidden = false;
-    m_notes = M_realnotes();
-    GetDlgItem(IDC_NOTES)->Invalidate();
+
+    // Swap windows
+    m_ex_notes.ShowWindow(SW_SHOW);
+    m_ex_notes.EnableWindow(TRUE);
+    m_ex_hidden_notes.ShowWindow(SW_HIDE);
+    m_ex_hidden_notes.EnableWindow(FALSE);
+
+    m_ex_notes.SetFocus();
+
     SetZoomMenu();
   }
 }
 
-void CAddEdit_Basic::HideNotes()
+void CAddEdit_Basic::HideNotes(const bool bForceHide)
 {
-  if (!m_isNotesHidden) { // idempotent
+  if (bForceHide || !m_isNotesHidden) {
     m_isNotesHidden = true;
-    M_realnotes() = m_notes;
-    
-    m_notes = HIDDEN_NOTES;
-    GetDlgItem(IDC_NOTES)->Invalidate();
-    // Disable zoom of hidden text
-    m_ex_notes.EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
-    m_ex_notes.EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
+
+    // Swap windows
+    m_ex_notes.ShowWindow(SW_HIDE);
+    m_ex_notes.EnableWindow(FALSE);
+    m_ex_hidden_notes.ShowWindow(SW_SHOW);
+    m_ex_hidden_notes.EnableWindow(TRUE);
   }
 }
 
@@ -1025,7 +1054,7 @@ void CAddEdit_Basic::OnGroupComboChanged()
 
 void CAddEdit_Basic::OnChanged()
 {
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0)
     return;
 
   UpdateData(TRUE);
@@ -1034,14 +1063,18 @@ void CAddEdit_Basic::OnChanged()
 
 void CAddEdit_Basic::OnENChangeNotes()
 {
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0)
     return;
 
   // Called for any change - even just clicking on it - so check really changed
-  CSecString current_notes;
-  m_ex_notes.GetWindowText(current_notes);
-  if (current_notes == M_realnotes()  || current_notes == HIDDEN_NOTES)
+  if (m_ex_hidden_notes.IsWindowVisible()) {
     return;
+  } else {
+    CSecString current_notes;
+    m_ex_notes.GetWindowText(current_notes);
+    if (current_notes == M_notes())
+      return;
+  }
 
   m_ae_psh->SetChanged(true);
   m_ae_psh->SetNotesChanged(true); // Needed if Notes field is long and will be truncated
@@ -1093,7 +1126,7 @@ void CAddEdit_Basic::OnSTCExClicked(UINT nID)
       break;
     case IDC_STATIC_NOTES:
       m_stc_notes.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = M_realnotes();
+      cs_data = M_notes();
       iaction = CItemData::NOTES;
       break;
     case IDC_STATIC_URL:
@@ -1132,38 +1165,24 @@ LRESULT CAddEdit_Basic::OnWordWrap(WPARAM, LPARAM)
 
   m_ex_notes.UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
 
+  // No idea why this is necessary but fixes the issue of no horizontaol
+  // scroll bar active after turning off Word Wrap if the preferences
+  // are Word Wrap and Hidden Notes. Without this "fix", user would have to
+  // ensure that the Notes field looses focus by clicking on another field.
+  m_ex_notes.EnableWindow(FALSE);
+  m_ex_notes.EnableWindow(TRUE);
+
+  if (m_isNotesHidden)
+    ShowNotes();
+
   return 0L;
 }
 
-void CAddEdit_Basic::OnENSetFocusNotes()
-{
-  UpdateData(TRUE);
-  ShowNotes();
-  UpdateData(FALSE);
-
-  // Try to put window back as it was before hidden
-  if (m_NotesFirstVisibleLine > 0) {
-    // Then scroll correctly
-    int nNewFirstVisibleLine = m_ex_notes.GetFirstVisibleLine();
-    pws_os::Trace(L"SetFocus: NewFirstLine: %d\n", nNewFirstVisibleLine);
-    if (m_NotesFirstVisibleLine != nNewFirstVisibleLine)
-      m_ex_notes.LineScroll(m_NotesFirstVisibleLine - nNewFirstVisibleLine, 0);
-  }
-}
-
 void CAddEdit_Basic::OnENKillFocusNotes()
 {
-  UpdateData(TRUE);
-
-  // Save first visible line
-  m_NotesFirstVisibleLine = m_isNotesHidden ? -1 : m_ex_notes.GetFirstVisibleLine();
-
   if (!PWSprefs::GetInstance()->GetPref(PWSprefs::ShowNotesDefault)) {
     HideNotes();
-  } else
-    M_realnotes() = m_notes;
-
-  UpdateData(FALSE);
+  }
 }
 
 void CAddEdit_Basic::OnLaunch()
@@ -1185,7 +1204,7 @@ void CAddEdit_Basic::OnLaunch()
                                                        M_username(),
                                                        sPassword,
                                                        sLastPassword,
-                                                       M_realnotes(),
+                                                       M_notes(),
                                                        M_URL(),
                                                        M_email(),
                                                        vactionverboffsets);
@@ -1227,10 +1246,12 @@ LRESULT CAddEdit_Basic::OnCallExternalEditor(WPARAM, LPARAM)
   m_bOKSave = GetParent()->GetDlgItem(IDOK)->EnableWindow(FALSE);
   m_bOKCancel = GetParent()->GetDlgItem(IDCANCEL)->EnableWindow(FALSE);
 
-  const CString cs_text(MAKEINTRESOURCE(IDS_NOTES_IN_EXTERNAL_EDITOR));
-  CWnd *pwnotes = GetDlgItem(IDC_NOTES);
-  pwnotes->EnableWindow(FALSE);
-  pwnotes->SetWindowText(cs_text);
+  m_ex_hidden_notes.SetWindowText(CS_EXTERNAL_EDITOR);
+  HideNotes(true);
+  m_bUsingNotesExternalEditor = true;
+
+  // Clear events
+  ghEvents[0] = ghEvents[1] = NULL;
 
   m_thread = CExtThread::BeginThread(ExternalEditorThread, this);
   return 0L;
@@ -1244,6 +1265,8 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   wchar_t lpPathBuffer[4096];
   DWORD dwBufSize(4096);
 
+  StringX sxEditorCmdLineParms = PWSprefs::GetInstance()->GetPref(PWSprefs::AltNotesEditorCmdLineParms);
+
   StringX sxEditor = PWSprefs::GetInstance()->GetPref(PWSprefs::AltNotesEditor);
   if (sxEditor.empty()) {
     // Find out the users default editor for "txt" files
@@ -1255,9 +1278,9 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
       CGeneralMsgBox gmb;
       gmb.AfxMessageBox(L"oops");
 #endif
-      // Send error return (WPARAM != 0)
-      self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
-      return 16;
+      self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 8, 0);
+      self->ResetHiddenNotes();
+      return 8;
     }
     sxEditor = szExecName;
   }
@@ -1269,9 +1292,9 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
     cs_msg.Format(IDS_CANT_FIND_EXT_EDITOR, sxEditor.c_str());
     gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONEXCLAMATION);
 
-    // Send error return (WPARAM != 0)
-    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
-    return 0;
+    self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 12, 0);
+    self->ResetHiddenNotes();
+    return 12;
   }
 
   sxEditor = szExecName;
@@ -1282,9 +1305,9 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
     cs_msg.Format(IDS_CANT_FIND_EXT_EDITOR, sxEditor.c_str());
     gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONEXCLAMATION);
 
-    // Send error return (WPARAM != 0)
-    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
-    return 0;
+    self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
+    self->ResetHiddenNotes();
+    return 16;
   }
 
   // Now we know the editor exists - go copy the data for it!
@@ -1301,7 +1324,9 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   FILE *fd;
 
   if ((fd = pws_os::FOpen(self->m_szTempName, L"w+b")) == NULL) {
-    return 0;
+    self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 20, 0);
+    self->ResetHiddenNotes();
+    return 20;
   }
 
   // Write BOM
@@ -1309,8 +1334,8 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   putwc(iBOM, fd);
 
   // Write out text
-  fwrite(reinterpret_cast<const void *>((LPCWSTR)self->M_realnotes()), sizeof(BYTE),
-             self->M_realnotes().GetLength() * sizeof(wchar_t), fd);
+  fwrite(reinterpret_cast<const void *>((LPCWSTR)self->M_notes()), sizeof(BYTE),
+             self->M_notes().GetLength() * sizeof(wchar_t), fd);
 
   // Close file before invoking editor
   fclose(fd);
@@ -1323,13 +1348,20 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   si.cb = sizeof(si);
   ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
 
-  DWORD dwCreationFlags(0);
-  dwCreationFlags = CREATE_UNICODE_ENVIRONMENT;
+  DWORD dwCreationFlags;
+  dwCreationFlags = CREATE_UNICODE_ENVIRONMENT | CREATE_SUSPENDED;
 
   CString cs_CommandLine;
 
-  // Make the command line = "<program>" "file"
-  cs_CommandLine.Format(L"\"%s\" \"%s\"", sxEditor.c_str(), self->m_szTempName);
+  // Make the command line = "<program>" <parameters> "filename"
+  // Note: the parameters reproduce the user's input as-is - it is up to them
+  // to add quotes or not as required by their chosen external editor.
+  // We add double quotes around the full path to the program and its name and
+  // similarly for the temporary file name e.g.
+  // "C:\somewhere\editor.exe" paramters "C:\somewhere_else\temp.txt"
+  cs_CommandLine.Format(L"\"%s\" %s \"%s\"", sxEditor.c_str(),
+                               sxEditorCmdLineParms.c_str(), self->m_szTempName);
+
   int ilen = cs_CommandLine.GetLength();
   LPWSTR pszCommandLine = cs_CommandLine.GetBuffer(ilen);
 
@@ -1343,31 +1375,61 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
     _wremove(self->m_szTempName);
     SecureZeroMemory(self->m_szTempName, sizeof(self->m_szTempName));
 
-    // Send error return (WPARAM != 0)
-    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
-    return 0;
+    self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 24, 0);
+    self->ResetHiddenNotes();
+    return 24;
   }
 
+  // All set up now set process going
+  ResumeThread(pi.hThread);
+
   WaitForInputIdle(pi.hProcess, INFINITE);
 
-  // Wait until child process exits.
-  WaitForSingleObject(pi.hProcess, INFINITE);
+  // Disable me to stop other changes until editor ends or user cancels it
+  self->EnableWindow(FALSE);
+
+  // Wait until child process exits or we cancel it
+  ghEvents[0] = pi.hProcess;                            // Thread ended
+  ghEvents[1] = CreateEvent(NULL, FALSE, FALSE, NULL);  // We cancelled
+
+  // Now wait for editor to end or user to cancel
+  DWORD dwEvent = WaitForMultipleObjects(2, ghEvents, FALSE, INFINITE);
 
   // Close process and thread handles.
   CloseHandle(pi.hProcess);
   CloseHandle(pi.hThread);
   cs_CommandLine.ReleaseBuffer();
 
-  self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 0, 0);
+  // If edit ended normally (i.e. not cancelled by user) - go process
+  // Otherwise indicate cancelled
+  self->SendMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, dwEvent == (WAIT_OBJECT_0 + 0) ? 0 : 28, 0);
+
+  self->ResetHiddenNotes();
   return 0;
 }
 
+void CAddEdit_Basic::ResetHiddenNotes()
+{
+  // Put back hidden notes message
+  m_ex_hidden_notes.SetWindowText(CS_HIDDEN_NOTES);
+  
+  // Show notes again
+  ShowNotes(true);
+
+  // No longer in the external editor
+  m_bUsingNotesExternalEditor = false;
+
+  // Re-enable the property page
+  EnableWindow(TRUE);
+}
+
 LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
 {
-  std::wstring note;
-  GetDlgItem(IDC_NOTES)->EnableWindow(TRUE);
+  std::wstring sNewNotes;
+  CSecString sOldNotes = M_notes();
 
   if (wParam != 0) {
+    // Tidy up and re-enable sheet OK/Cancel buttons
     goto error_exit;
   }
 
@@ -1378,7 +1440,7 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
     goto error_exit;
   }
 
-  M_realnotes().Empty();
+  M_notes().Empty();
 
   ulong64 flength = pws_os::fileLength(fd);
 
@@ -1413,15 +1475,16 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
   // Read in text
   fread(pBuffer, sizeof(BYTE), slength - 2, fd);
 
-  note = reinterpret_cast<const LPCWSTR>(pBuffer);
+  sNewNotes = reinterpret_cast<const LPCWSTR>(pBuffer);
 
   delete [] pBuffer;
 
   // Close file before invoking editor
   fclose(fd);
 
-  if (note.length() > MAXTEXTCHARS) {
-    note = note.substr(0, MAXTEXTCHARS);
+  if ((!M_pcore()->IsReadOnly() && M_protected() == 0) &&
+      sNewNotes.length() > MAXTEXTCHARS) {
+    sNewNotes = sNewNotes.substr(0, MAXTEXTCHARS);
 
     CGeneralMsgBox gmb;
     CString cs_text, cs_title(MAKEINTRESOURCE(IDS_WARNINGTEXTLENGTH));
@@ -1431,10 +1494,10 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
 
   // Set real notes field, and
   // we are still displaying the old text, so replace that too
-  M_realnotes() = m_notes = note.c_str();
+  M_notes() = sNewNotes.c_str();
 
   UpdateData(FALSE);
-  GetDlgItem(IDC_NOTES)->Invalidate();
+  m_ex_notes.Invalidate();
 
   // Delete temporary file
   _wremove(m_szTempName);
@@ -1444,15 +1507,16 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
   GetParent()->GetDlgItem(IDOK)->EnableWindow(m_bOKSave == 0 ? TRUE : FALSE);
   GetParent()->GetDlgItem(IDCANCEL)->EnableWindow(m_bOKCancel == 0 ? TRUE : FALSE);
 
-  OnENChangeNotes();
+  if (sOldNotes != M_notes()) {
+    m_ae_psh->SetChanged(true);
+    m_ae_psh->SetNotesChanged(true); // Needed if Notes field is long and will be truncated
+  }
   return 0L;
 
 error_exit:
-  // Restore notes field and get out
-  m_notes = M_realnotes();
-
+  // Get out
   UpdateData(FALSE);
-  GetDlgItem(IDC_NOTES)->Invalidate();
+  m_ex_notes.Invalidate();
 
   // Restore Sheet buttons
   GetParent()->GetDlgItem(IDOK)->EnableWindow(m_bOKSave == 0 ? TRUE : FALSE);
@@ -1460,25 +1524,6 @@ error_exit:
   return 0L;
 }
 
-void CAddEdit_Basic::OnViewDependents()
-{
-  // Ignore unless button is visible
-  if (!GetDlgItem(IDC_VIEWDEPENDENTS)->IsWindowVisible())
-    return;
-
-  CString cs_msg, cs_type;
-  CGeneralMsgBox gmb;
-
-  if (M_original_entrytype() == CItemData::ET_ALIASBASE)
-    cs_type.LoadString(M_num_dependents() == 1 ? IDSC_ALIAS : IDSC_ALIASES);
-  else
-    cs_type.LoadString(M_num_dependents() == 1 ? IDSC_SHORTCUT : IDSC_SHORTCUTS);
-
-  cs_msg.Format(IDS_VIEWDEPENDENTS, M_num_dependents(), static_cast<LPCWSTR>(cs_type),
-                static_cast<LPCWSTR>(M_dependents()));
-  gmb.MessageBox(cs_msg, AfxGetAppName(), MB_OK);
-}
-
 bool CAddEdit_Basic::CheckNewPassword(const StringX &group, const StringX &title,
                                       const StringX &user, const StringX &password,
                                       const bool bIsEdit, const CItemData::EntryType InputType, 
@@ -1676,3 +1721,61 @@ void CAddEdit_Basic::OnCopyPassword()
   GetMainDlg()->SetClipboardData(m_password);
   GetMainDlg()->UpdateLastClipboardAction(CItemData::PASSWORD);
 }
+
+void CAddEdit_Basic::SetUpDependentsCombo()
+{
+  // Get Add/Edit font
+  CFont *pFont = Fonts::GetInstance()->GetAddEditFont();
+  m_cmbDependents.SetFont(pFont);
+
+  CString cs_type;
+  cs_type.LoadString(M_original_entrytype() == CItemData::ET_ALIASBASE ?
+      IDS_LIST_OF_ALIASES : IDS_LIST_OF_SHORTCUTS);
+
+  m_stc_dependent.SetWindowText(cs_type);
+
+  for (size_t i = 0; i < M_vsxdependents().size(); i++) {
+    m_cmbDependents.AddString(M_vsxdependents()[i].c_str());
+  }
+
+  SetComboBoxWidth();
+}
+
+void CAddEdit_Basic::SetComboBoxWidth()
+{
+  // Find the longest string in the combo box.
+  CString str;
+  CSize sz;
+  int dx = 0;
+  TEXTMETRIC tm;
+  CDC *pDC = m_cmbDependents.GetDC();
+  CFont *pFont = m_cmbDependents.GetFont();
+
+  // Select the listbox font, save the old font
+  CFont *pOldFont = pDC->SelectObject(pFont);
+
+  // Get the text metrics for avg char width
+  pDC->GetTextMetrics(&tm);
+
+  for (int i = 0; i < m_cmbDependents.GetCount(); i++) {
+    m_cmbDependents.GetLBText(i, str);
+    sz = pDC->GetTextExtent(str);
+
+    // Add the avg width to prevent clipping
+    sz.cx += tm.tmAveCharWidth;
+
+    if (sz.cx > dx)
+      dx = sz.cx;
+  }
+
+  // Select the old font back into the DC
+  pDC->SelectObject(pOldFont);
+  m_cmbDependents.ReleaseDC(pDC);
+
+  // Adjust the width for the vertical scroll bar and the left and right border.
+  dx += ::GetSystemMetrics(SM_CXVSCROLL) + 2 * ::GetSystemMetrics(SM_CXEDGE);
+
+  // If the width of the list box is too small, adjust it so that every
+  // item is completely visible.
+  m_cmbDependents.SetDroppedWidth(dx);
+}
