@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -108,7 +108,7 @@ static void AFXAPI DDV_CheckMaxDays(CDataExchange* pDX, const int &how,
   }
 }
 
-BOOL CAddEdit_DateTimes::PreTranslateMessage(MSG* pMsg)
+BOOL CAddEdit_DateTimes::PreTranslateMessage(MSG *pMsg)
 {
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_F1) {
     PostMessage(WM_COMMAND, MAKELONG(ID_HELP, BN_CLICKED), NULL);
@@ -209,7 +209,7 @@ BOOL CAddEdit_DateTimes::OnInitDialog()
   // Refresh dialog
   m_bInitdone = true;
   UpdateStats();
-  return TRUE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 BOOL CAddEdit_DateTimes::OnKillActive()
@@ -297,7 +297,7 @@ void CAddEdit_DateTimes::UpdateStats()
     return;
 
   CString cs_text;
-  cs_text.Format(L"%u", M_entrysize());
+  cs_text.Format(L"%lu", M_entrysize());
 
   for (int i = cs_text.GetLength() - 3; i > 0; i -= 3) {
     cs_text.Insert(i, L",");
@@ -330,7 +330,7 @@ LRESULT CAddEdit_DateTimes::OnQuerySiblings(WPARAM wParam, LPARAM )
       break;
     case PP_UPDATE_VARIABLES:
       // Since OnOK calls OnApply after we need to verify and/or
-      // copy data into the entry - we do it ourselfs here first
+      // copy data into the entry - we do it ourselves here first
       if (OnApply() == FALSE)
         return 1L;
       break;
@@ -362,7 +362,7 @@ BOOL CAddEdit_DateTimes::OnApply()
 void CAddEdit_DateTimes::OnDateTimeChanged(NMHDR *, LRESULT *pLResult)
 {
   *pLResult = 0;
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY || m_inSetX)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0 || m_inSetX)
     return;
 
   SetXTime();
@@ -370,7 +370,7 @@ void CAddEdit_DateTimes::OnDateTimeChanged(NMHDR *, LRESULT *pLResult)
 
 void CAddEdit_DateTimes::OnDaysChanged()
 {
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY || m_inSetX)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0 || m_inSetX)
     return;
 
   SetXTime();
@@ -397,6 +397,7 @@ void CAddEdit_DateTimes::OnHowChanged()
     GetDlgItem(IDC_STATIC_LTINTERVAL_NOW)->EnableWindow(FALSE);
     GetDlgItem(IDC_REUSE_ON_CHANGE)->EnableWindow(FALSE);
     GetDlgItem(IDC_EXPIRYDATE)->EnableWindow(FALSE);
+    GetDlgItem(IDC_EXPDAYSSPIN)->EnableWindow(FALSE);
     break;
   case ABSOLUTE_EXP:
     m_ae_psh->SetChanged(true);
@@ -405,21 +406,23 @@ void CAddEdit_DateTimes::OnHowChanged()
     GetDlgItem(IDC_STATIC_LTINTERVAL_NOW)->EnableWindow(FALSE);
     GetDlgItem(IDC_REUSE_ON_CHANGE)->EnableWindow(FALSE);
     GetDlgItem(IDC_EXPIRYDATE)->EnableWindow(TRUE);
+    GetDlgItem(IDC_EXPDAYSSPIN)->EnableWindow(FALSE);
     break;
   case RELATIVE_EXP:
     m_ae_psh->SetChanged(true);
     SetXTime();
-GetDlgItem(IDC_EXPDAYS)->EnableWindow(TRUE);
+    GetDlgItem(IDC_EXPDAYS)->EnableWindow(TRUE);
     GetDlgItem(IDC_STATIC_LTINTERVAL_NOW)->EnableWindow(TRUE);
     GetDlgItem(IDC_REUSE_ON_CHANGE)->EnableWindow(TRUE);
     GetDlgItem(IDC_EXPIRYDATE)->EnableWindow(FALSE);
+    GetDlgItem(IDC_EXPDAYSSPIN)->EnableWindow(TRUE);
     break;
   default:
     ASSERT(0);
   }
+  Invalidate();
 }
 
-
 void CAddEdit_DateTimes::SetXTime()
 {
   m_inSetX = true;
@@ -461,7 +464,7 @@ void CAddEdit_DateTimes::OnRecurringPswdExpiry()
   // (should suffice for most purposes). For non-recurring, limit is
   // the max that won't overflow time_t
   const int new_max = (m_bRecurringPswdExpiry == TRUE) ? 3650 : m_maxDays;
-  CSpinButtonCtrl* pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_EXPDAYSSPIN);
+  CSpinButtonCtrl *pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_EXPDAYSSPIN);
   pspin->SetRange32(1, new_max);
   if (m_numDays > new_max)
     m_numDays = 1;
