@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -16,6 +16,7 @@
 #include "AddEdit_PasswordPolicy.h"
 #include "AddEdit_PropertySheet.h"
 #include "GeneralMsgBox.h"
+#include "Fonts.h"
 
 #include "core/core.h"
 #include "core/PwsPlatform.h"
@@ -23,6 +24,8 @@
 #include "core/PWSprefs.h"
 #include "core/PWCharPool.h"
 
+#include <algorithm>
+
 #include "resource.h"
 #include "resource3.h"  // String resources
 
@@ -57,8 +60,7 @@ CAddEdit_PasswordPolicy::CAddEdit_PasswordPolicy(CWnd *pParent,
                                                  st_AE_master_data *pAEMD)
   : CAddEdit_PropertyPage(pParent,
                           CAddEdit_PasswordPolicy::IDD, CAddEdit_PasswordPolicy::IDD_SHORT,
-                          pAEMD),
-    m_useownsymbols(DEFAULT_SYMBOLS), m_bInitdone(false)
+                          pAEMD), m_bInitdone(false)
 {
   m_policy_radibtn = M_ipolicy() != SPECIFIC_POLICY ? 0 : 1 ;
 
@@ -114,7 +116,6 @@ void CAddEdit_PasswordPolicy::DoDataExchange(CDataExchange* pDX)
   DDX_Check(pDX, IDC_USEHEXDIGITS, m_pwusehexdigits);
   DDX_Check(pDX, IDC_PRONOUNCEABLE, m_pwmakepronounceable);
 
-  DDX_Radio(pDX, IDC_USEDEFAULTSYMBOLS, m_useownsymbols);
   DDX_Control(pDX, IDC_OWNSYMBOLS, (CEdit&)m_symbols);
   
   DDX_Control(pDX, IDC_POLICYLIST, m_cbxPolicyNames);
@@ -134,8 +135,6 @@ BEGIN_MESSAGE_MAP(CAddEdit_PasswordPolicy, CAddEdit_PropertyPage)
   ON_BN_CLICKED(IDC_PRONOUNCEABLE, OnMakePronounceable)
   ON_BN_CLICKED(IDC_USEPWPOLICYNAME, OnSelectNamedPolicy)
   ON_BN_CLICKED(IDC_ENTRYPWPOLICY, OnSetSpecificPWPolicy)
-  ON_BN_CLICKED(IDC_USEDEFAULTSYMBOLS, OnSymbols)
-  ON_BN_CLICKED(IDC_USEOWNSYMBOLS, OnSymbols)
 
   ON_EN_CHANGE(IDC_DEFPWLENGTH, OnChanged)
   ON_EN_CHANGE(IDC_MINDIGITLENGTH, OnChanged)
@@ -150,12 +149,13 @@ BEGIN_MESSAGE_MAP(CAddEdit_PasswordPolicy, CAddEdit_PropertyPage)
   // Common
   ON_MESSAGE(PSM_QUERYSIBLINGS, OnQuerySiblings)
   //}}AFX_MSG_MAP
+  ON_BN_CLICKED(IDC_RESET_SYMBOLS, &CAddEdit_PasswordPolicy::OnSymbolReset)
 END_MESSAGE_MAP()
 
 /////////////////////////////////////////////////////////////////////////////
 // CAddEdit_PasswordPolicy message handlers
 
-BOOL CAddEdit_PasswordPolicy::PreTranslateMessage(MSG* pMsg)
+BOOL CAddEdit_PasswordPolicy::PreTranslateMessage(MSG *pMsg)
 {
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_F1) {
     PostMessage(WM_COMMAND, MAKELONG(ID_HELP, BN_CLICKED), NULL);
@@ -165,10 +165,30 @@ BOOL CAddEdit_PasswordPolicy::PreTranslateMessage(MSG* pMsg)
   return CAddEdit_PropertyPage::PreTranslateMessage(pMsg);
 }
 
+// Following is also in PasswordPolicDlg.cpp. Move to common mixin?
+static void setupBuddy(CWnd *p, int spinid, int id, int &length, PWSprefs::IntPrefs iPref)
+{
+  const int minValue = PWSprefs::GetInstance()->GetPrefMinVal(iPref);
+  const int maxValue = PWSprefs::GetInstance()->GetPrefMaxVal(iPref);
+
+  CSpinButtonCtrl *pspin = (CSpinButtonCtrl *)p->GetDlgItem(spinid);
+  pspin->SetBuddy(p->GetDlgItem(id));
+  pspin->SetRange32(minValue, maxValue);
+  pspin->SetBase(10);
+  pspin->SetPos(length);
+}
+
 BOOL CAddEdit_PasswordPolicy::OnInitDialog()
 {
   CAddEdit_PropertyPage::OnInitDialog();
 
+  // Get Add/Edit font
+  Fonts *pFonts = Fonts::GetInstance();
+  CFont *pFont = pFonts->GetAddEditFont();
+
+  // Change font size of the user supplied symbol fields
+  m_symbols.SetFont(pFont);
+
   // Populate the combo box
   m_cbxPolicyNames.ResetContent();
   m_cbxPolicyNames.ChangeColour();
@@ -200,17 +220,15 @@ BOOL CAddEdit_PasswordPolicy::OnInitDialog()
   m_cbxPolicyNames.GetComboBoxInfo(&info);
   ::SendMessage(info.hwndItem ,EM_SETREADONLY, TRUE, 0);
 
-  if (!vNames.empty()) {
-    if (M_uicaller() == IDS_VIEWENTRY || M_protected() != 0) {
-      // Read-only
-      m_cbxPolicyNames.EnableWindow(FALSE);
-      m_cbxPolicyNames.ShowWindow(M_ipolicy() == SPECIFIC_POLICY ? SW_HIDE : SW_SHOW);
-    }
+  if (M_uicaller() == IDS_VIEWENTRY || M_protected() != 0) {
+    // Read-only
+    m_cbxPolicyNames.EnableWindow(FALSE);
+    m_cbxPolicyNames.ShowWindow(M_ipolicy() == SPECIFIC_POLICY ? SW_HIDE : SW_SHOW);
+  } else {
+    // If specific policy - disable named policy combobox
+    m_cbxPolicyNames.EnableWindow(M_ipolicy() == SPECIFIC_POLICY ? FALSE : TRUE);
   }
 
-  // If specific policy - disable named policy comboboc
-  m_cbxPolicyNames.EnableWindow(M_ipolicy() == SPECIFIC_POLICY ? FALSE : TRUE);
-
   if (M_ipolicy() == NAMED_POLICY) {
     if (index != 0) {
       PWPolicy st_pp;
@@ -226,37 +244,11 @@ BOOL CAddEdit_PasswordPolicy::OnInitDialog()
   }
 
   // Set up spin control relationships
-  CSpinButtonCtrl* pspin;
-
-  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_PWLENSPIN);
-  pspin->SetBuddy(GetDlgItem(IDC_DEFPWLENGTH));
-  pspin->SetRange(4, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos((int)m_pwdefaultlength);
-
-  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINDIGITS);
-  pspin->SetBuddy(GetDlgItem(IDC_MINDIGITLENGTH));
-  pspin->SetRange(0, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos((int)m_pwdigitminlength);
-
-  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINLOWERCASE);
-  pspin->SetBuddy(GetDlgItem(IDC_MINLOWERLENGTH));
-  pspin->SetRange(0, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos((int)m_pwlowerminlength);
-
-  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINSYMBOLS);
-  pspin->SetBuddy(GetDlgItem(IDC_MINSYMBOLLENGTH));
-  pspin->SetRange(0, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos((int)m_pwsymbolminlength);
-
-  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINUPPERCASE);
-  pspin->SetBuddy(GetDlgItem(IDC_MINUPPERLENGTH));
-  pspin->SetRange(0, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos((int)m_pwupperminlength);
+  setupBuddy(this, IDC_PWLENSPIN, IDC_DEFPWLENGTH, m_pwdefaultlength, PWSprefs::PWDefaultLength);
+  setupBuddy(this, IDC_SPINDIGITS, IDC_MINDIGITLENGTH, m_pwdigitminlength, PWSprefs::PWDigitMinLength);
+  setupBuddy(this, IDC_SPINLOWERCASE, IDC_MINLOWERLENGTH, m_pwlowerminlength, PWSprefs::PWLowercaseMinLength);
+  setupBuddy(this, IDC_SPINUPPERCASE, IDC_MINUPPERLENGTH, m_pwupperminlength, PWSprefs::PWUppercaseMinLength);
+  setupBuddy(this, IDC_SPINSYMBOLS, IDC_MINSYMBOLLENGTH, m_pwsymbolminlength, PWSprefs::PWSymbolMinLength);
 
   // Disable controls based on m_ipolicy
   SetPolicyControls();
@@ -273,28 +265,20 @@ BOOL CAddEdit_PasswordPolicy::OnInitDialog()
     DisablePolicy();
     GetDlgItem(IDC_USEPWPOLICYNAME)->EnableWindow(FALSE);
     GetDlgItem(IDC_ENTRYPWPOLICY)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_OR)->EnableWindow(FALSE);
     GetDlgItem(IDC_STATIC_PWLEN)->EnableWindow(FALSE);
+    GetDlgItem(IDC_STATIC_OR)->EnableWindow(FALSE);
   }
 
   // Setup symbols
-  StringX sx_symbols = PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultSymbols);
-  if (sx_symbols.length() == 0) {
-    stringT st_symbols;
-    CPasswordCharPool::GetDefaultSymbols(st_symbols);
-    sx_symbols = st_symbols.c_str();
-  }
-
-  GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->SetWindowText(sx_symbols.c_str());
   m_symbols.SetWindowText(M_symbols());
 
   m_bInitdone = true;
-  return TRUE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 void CAddEdit_PasswordPolicy::OnChanged()
 {
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0)
     return;
 
   UpdateData(TRUE);
@@ -339,30 +323,34 @@ bool CAddEdit_PasswordPolicy::ValidatePolicy(CWnd *&pFocus)
     return false;
   }
 
-  if ((m_pwdefaultlength < 4) || (m_pwdefaultlength > 1024)) {
-    gmb.AfxMessageBox(IDS_DEFAULTPWLENGTH);
+  if ((m_pwdefaultlength < M_prefminPWLength()) || (m_pwdefaultlength > M_prefmaxPWLength())) {
+    CString csText;
+    csText.Format(IDS_DEFAULTPWLENGTH, M_prefminPWLength(), M_prefmaxPWLength());
+    gmb.AfxMessageBox(csText);
     pFocus = GetDlgItem(IDC_DEFPWLENGTH);
     return false;
   }
 
-  if (!(m_pwusehexdigits || m_pweasyvision || m_pwmakepronounceable) &&
-      (m_pwdigitminlength + m_pwlowerminlength +
-       m_pwsymbolminlength + m_pwupperminlength) > m_pwdefaultlength) {
+  if (!(m_pwusehexdigits || m_pwmakepronounceable) &&
+      ((m_pwusedigits ? m_pwdigitminlength : 0) +
+       (m_pwuselowercase ? m_pwlowerminlength : 0) +
+       (m_pwusesymbols ? m_pwsymbolminlength : 0) +
+       (m_pwuseuppercase ? m_pwupperminlength : 0)) > m_pwdefaultlength) {
     gmb.AfxMessageBox(IDS_DEFAULTPWLENGTHTOOSMALL);
     pFocus = GetDlgItem(IDC_DEFPWLENGTH);
     return false;
   }
 
-  if ((m_pwusehexdigits || m_pweasyvision || m_pwmakepronounceable))
-    m_pwdigitminlength = m_pwlowerminlength =
-      m_pwsymbolminlength = m_pwupperminlength = 1;
-
   return true;
 }
 
 BOOL CAddEdit_PasswordPolicy::OnKillActive()
 {
+  if (UpdateData(TRUE) == FALSE)
+    return FALSE;
+
   CWnd *pFocus(NULL);
+
   if (ValidatePolicy(pFocus)) {
     SetPolicyFromVariables();
   } else {
@@ -454,9 +442,7 @@ void CAddEdit_PasswordPolicy::do_hex(const bool bHex)
       GetDlgItem(LenTxts[i * 2 + 1])->EnableWindow(FALSE);
     }
 
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(FALSE);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(FALSE);
     GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(FALSE);
   } else {
     // non-hex, restore state
@@ -476,18 +462,15 @@ void CAddEdit_PasswordPolicy::do_hex(const bool bHex)
 
     BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED &&
                    m_pweasyvision == FALSE && m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bEnable == TRUE && m_useownsymbols == OWN_SYMBOLS) ?
-                                             TRUE : FALSE);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnable);
+    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnable == TRUE);
   }
 }
 
 void CAddEdit_PasswordPolicy::do_easyorpronounceable(const bool bSet)
 {
   // Can't have any minimum lengths set!
-  if ((m_pweasyvision == TRUE  || m_pwmakepronounceable == TRUE) &&
+  if ((m_pwmakepronounceable == TRUE) &&
       (m_pwdigitminlength  > 1 || m_pwlowerminlength > 1 ||
        m_pwsymbolminlength > 1 || m_pwupperminlength > 1)) {
     CGeneralMsgBox gmb;
@@ -497,134 +480,74 @@ void CAddEdit_PasswordPolicy::do_easyorpronounceable(const bool bSet)
   CString cs_value;
   int i;
   if (bSet) {
-    // Hide lengths
+    // Disable lengths
     for (i = 0; i < N_HEX_LENGTHS; i++) {
       m_save_enabled[i][0] = GetDlgItem(nonHexLengths[i])->IsWindowEnabled();
       m_save_visible[i] = GetDlgItem(nonHexLengths[i])->IsWindowVisible();
       GetDlgItem(nonHexLengths[i])->EnableWindow(FALSE);
-      GetDlgItem(nonHexLengths[i])->ShowWindow(SW_HIDE);
       GetDlgItem(nonHexLengthSpins[i])->EnableWindow(FALSE);
-      GetDlgItem(nonHexLengthSpins[i])->ShowWindow(SW_HIDE);
       GetDlgItem(LenTxts[i * 2])->EnableWindow(FALSE);
-      GetDlgItem(LenTxts[i * 2])->ShowWindow(SW_HIDE);
       GetDlgItem(LenTxts[i * 2 + 1])->EnableWindow(FALSE);
-      GetDlgItem(LenTxts[i * 2 + 1])->ShowWindow(SW_HIDE);
     }
 
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow((IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED) ? TRUE : FALSE);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(FALSE);
   } else {
     // Show lengths
     for (i = 0; i < N_HEX_LENGTHS; i++) {
-      const int iShow = m_save_visible[i] == TRUE ? SW_SHOW : SW_HIDE;
       GetDlgItem(nonHexLengths[i])->EnableWindow(m_save_enabled[i][0]);
-      GetDlgItem(nonHexLengths[i])->ShowWindow(iShow);
       GetDlgItem(nonHexLengthSpins[i])->EnableWindow(m_save_enabled[i][0]);
-      GetDlgItem(nonHexLengthSpins[i])->ShowWindow(iShow);
       GetDlgItem(LenTxts[i * 2])->EnableWindow(m_save_enabled[i][0]);
-      GetDlgItem(LenTxts[i * 2])->ShowWindow(iShow);
       GetDlgItem(LenTxts[i * 2 + 1])->EnableWindow(m_save_enabled[i][0]);
-      GetDlgItem(LenTxts[i * 2 + 1])->ShowWindow(iShow);
     }
-
-    BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED) ? TRUE : FALSE;
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bEnable == TRUE && m_useownsymbols == OWN_SYMBOLS) ?
-                                             TRUE : FALSE);
   }
+  OnSymbolReset();
+}
+
+void CAddEdit_PasswordPolicy::do_useX(UseX x)
+{
+  const struct { int cb; int edit; int spin; int left; int right;} controls[] = {
+    {IDC_USELOWERCASE, IDC_MINLOWERLENGTH,  IDC_SPINLOWERCASE, IDC_STATIC_LC1, IDC_STATIC_LC2},
+    {IDC_USEUPPERCASE, IDC_MINUPPERLENGTH,  IDC_SPINUPPERCASE, IDC_STATIC_UC1, IDC_STATIC_UC2},
+    {IDC_USEDIGITS,    IDC_MINDIGITLENGTH,  IDC_SPINDIGITS,    IDC_STATIC_DG1, IDC_STATIC_DG2},
+    {IDC_USESYMBOLS,   IDC_MINSYMBOLLENGTH, IDC_SPINSYMBOLS,   IDC_STATIC_SY1, IDC_STATIC_SY2},
+  };
+
+  UpdateData(TRUE);
+
+  m_ae_psh->SetChanged(true);
+
+  BOOL bEnable = (IsDlgButtonChecked(controls[x].cb) == BST_CHECKED &&
+                  m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
+
+  GetDlgItem(controls[x].edit)->EnableWindow(bEnable);
+  GetDlgItem(controls[x].spin)->EnableWindow(bEnable);
+  GetDlgItem(controls[x].left)->EnableWindow(bEnable);
+  GetDlgItem(controls[x].right)->EnableWindow(bEnable);
+  if (x == USESYM) {
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnable);
+    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnable == TRUE);
+  }
+
+  UpdateData(FALSE);
 }
 
 void CAddEdit_PasswordPolicy::OnUseLowerCase()
 {
-  UpdateData(TRUE);
-  m_ae_psh->SetChanged(true);
-
-  BOOL bEnable = (IsDlgButtonChecked(IDC_USELOWERCASE) == BST_CHECKED &&
-                   m_pweasyvision == FALSE && m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
-  int iShow = (m_pweasyvision == TRUE || m_pwmakepronounceable == TRUE) ? SW_HIDE : SW_SHOW;
-
-  GetDlgItem(IDC_MINLOWERLENGTH)->EnableWindow(bEnable);
-  GetDlgItem(IDC_MINLOWERLENGTH)->ShowWindow(iShow);
-  GetDlgItem(IDC_SPINLOWERCASE)->EnableWindow(bEnable);
-  GetDlgItem(IDC_SPINLOWERCASE)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_LC1)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_LC1)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_LC2)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_LC2)->ShowWindow(iShow);
-  m_pwlowerminlength = IsDlgButtonChecked(IDC_USELOWERCASE);  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USELOWER);
 }
 
 void CAddEdit_PasswordPolicy::OnUseUpperCase()
 {
-  UpdateData(TRUE);
-  m_ae_psh->SetChanged(true);
-
-  BOOL bEnable = (IsDlgButtonChecked(IDC_USEUPPERCASE) == BST_CHECKED &&
-                   m_pweasyvision == FALSE && m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
-  int iShow = (m_pweasyvision == TRUE || m_pwmakepronounceable == TRUE) ? SW_HIDE : SW_SHOW;
-
-  GetDlgItem(IDC_MINUPPERLENGTH)->EnableWindow(bEnable);
-  GetDlgItem(IDC_MINUPPERLENGTH)->ShowWindow(iShow);
-  GetDlgItem(IDC_SPINUPPERCASE)->EnableWindow(bEnable);
-  GetDlgItem(IDC_SPINUPPERCASE)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_UC1)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_UC1)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_UC2)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_UC2)->ShowWindow(iShow);
-  m_pwupperminlength = IsDlgButtonChecked(IDC_USEUPPERCASE);  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USEUPPER);
 }
 
 void CAddEdit_PasswordPolicy::OnUseDigits()
 {
-  UpdateData(TRUE);
-  m_ae_psh->SetChanged(true);
-
-  BOOL bEnable = (IsDlgButtonChecked(IDC_USEDIGITS) == BST_CHECKED &&
-                   m_pweasyvision == FALSE && m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
-  int iShow = (m_pweasyvision == TRUE || m_pwmakepronounceable == TRUE) ? SW_HIDE : SW_SHOW;
-
-  GetDlgItem(IDC_MINDIGITLENGTH)->EnableWindow(bEnable);
-  GetDlgItem(IDC_MINDIGITLENGTH)->ShowWindow(iShow);
-  GetDlgItem(IDC_SPINDIGITS)->EnableWindow(bEnable);
-  GetDlgItem(IDC_SPINDIGITS)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_DG1)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_DG1)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_DG2)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_DG2)->ShowWindow(iShow);
-  m_pwdigitminlength = IsDlgButtonChecked(IDC_USEDIGITS);  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USEDIGITS);
 }
 
 void CAddEdit_PasswordPolicy::OnUseSymbols()
 {
-  UpdateData(TRUE);
-  m_ae_psh->SetChanged(true);
-
-  BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED &&
-                   m_pweasyvision == FALSE && m_pwmakepronounceable == FALSE) ? TRUE : FALSE;
-  int iShow = (m_pweasyvision == TRUE || m_pwmakepronounceable == TRUE) ? SW_HIDE : SW_SHOW;
-
-  GetDlgItem(IDC_MINSYMBOLLENGTH)->EnableWindow(bEnable);
-  GetDlgItem(IDC_MINSYMBOLLENGTH)->ShowWindow(iShow);
-  GetDlgItem(IDC_SPINSYMBOLS)->EnableWindow(bEnable);
-  GetDlgItem(IDC_SPINSYMBOLS)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_SY1)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_SY1)->ShowWindow(iShow);
-  GetDlgItem(IDC_STATIC_SY2)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_SY2)->ShowWindow(iShow);
-  GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bEnable);
-  GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bEnable);
-  GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow((IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED) ? TRUE : FALSE);
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bEnable == TRUE && m_useownsymbols == OWN_SYMBOLS) ?
-                                           TRUE : FALSE);
-  m_pwsymbolminlength = IsDlgButtonChecked(IDC_USESYMBOLS);  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USESYM);
 }
 
 void CAddEdit_PasswordPolicy::OnUseHexdigits()
@@ -648,7 +571,7 @@ void CAddEdit_PasswordPolicy::OnEasyVision()
     m_pweasyvision = FALSE;
   }
 
-  do_easyorpronounceable(IsDlgButtonChecked(IDC_EASYVISION) == BST_CHECKED);
+  OnSymbolReset();
   UpdateData(FALSE);
 }
 
@@ -668,24 +591,9 @@ void CAddEdit_PasswordPolicy::OnMakePronounceable()
   UpdateData(FALSE);
 }
 
-void BubbleSort(std::wstring &str)
-{
-  wchar_t tmp;
-
-  for (size_t i = 0; i < str.size() - 1; ++i) {
-    for (size_t j = i + 1; j < str.size(); ++j) {
-      if (str[i] < str[j]) {
-        tmp = str[i];
-        str[i] = str[j];
-        str[j] = tmp;
-      }
-    }
-  }
-}
-
 void CAddEdit_PasswordPolicy::OnOwnSymbolsChanged()
 {
-  if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY)
+  if (!m_bInitdone || M_uicaller() == IDS_VIEWENTRY || M_protected() != 0)
     return;
 
   UpdateData(TRUE);
@@ -703,11 +611,11 @@ void CAddEdit_PasswordPolicy::OnOwnSymbolsChanged()
     bIsSymbolsChanged = true;
   } else {
     // We do not care if same string in different order:
-    //  so Bubble sort new string
-    BubbleSort(newstr);
+    //  so sort new string
+    std::sort(newstr.begin(), newstr.end());
 
-    //  then Bubble sort old string
-    BubbleSort(oldstr);
+    //  then sort old string
+    std::sort(oldstr.begin(), oldstr.end());
 
     // Then compare
     bIsSymbolsChanged = oldstr.compare(newstr) != 0;
@@ -715,17 +623,6 @@ void CAddEdit_PasswordPolicy::OnOwnSymbolsChanged()
   m_ae_psh->SetSymbolsChanged(bIsSymbolsChanged);
 }
 
-void CAddEdit_PasswordPolicy::OnSymbols()
-{
-  m_ae_psh->SetChanged(true);
-  m_useownsymbols = ((CButton *)GetDlgItem(IDC_USEDEFAULTSYMBOLS))->GetCheck() == BST_CHECKED ?
-                      DEFAULT_SYMBOLS : OWN_SYMBOLS;
-
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(m_useownsymbols == DEFAULT_SYMBOLS ? FALSE : TRUE);
-  if (m_useownsymbols == OWN_SYMBOLS)
-    GetDlgItem(IDC_OWNSYMBOLS)->SetFocus();
-}
-
 void CAddEdit_PasswordPolicy::OnSelectNamedPolicy()
 {
   m_ae_psh->SetChanged(true);
@@ -829,18 +726,16 @@ void CAddEdit_PasswordPolicy::SetPolicyControls()
   GetDlgItem(IDC_USEHEXDIGITS)->EnableWindow(bEnableSpecificPolicy);
 
   // Deal with symbols...
-  // Default policy - all 4 disabled (2 x radio buttons, 2 x static text).
-  // Specific policy - if default, then enable radio buttons, default set text.
-  // Specific policy - if own symbols, then all 4 enabled.
-  BOOL bDefaultSymbolsAndRadioButtons(FALSE), bOwnSymbolsEditControl(FALSE);
-  if (m_pwusesymbols == TRUE && bEnableSpecificPolicy == TRUE) {
-     bDefaultSymbolsAndRadioButtons = TRUE;
-     bOwnSymbolsEditControl = (m_useownsymbols == OWN_SYMBOLS) ? TRUE : FALSE;
-  }
-  GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bDefaultSymbolsAndRadioButtons);
-  GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bDefaultSymbolsAndRadioButtons);
-  GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bDefaultSymbolsAndRadioButtons);
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bOwnSymbolsEditControl);
+  // Default policy - all disabled like the rest.
+  // Specific policy - enable if Use Symbols set.
+  BOOL bEnableSymControls = (m_pwusesymbols == TRUE && bEnableSpecificPolicy == TRUE) ? TRUE: FALSE;
+
+  GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnableSymControls);
+  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnableSymControls);
+
+  // populate m_symbols
+  if (M_symbols().IsEmpty())
+    OnSymbolReset();
 
   UpdateData(FALSE);
 }
@@ -886,13 +781,12 @@ void CAddEdit_PasswordPolicy::SetPolicyFromVariables()
       M_pwp().symbolminlength = (int)m_pwsymbolminlength;
       M_pwp().upperminlength = (int)m_pwupperminlength;
 
-      if (m_pwusesymbols == TRUE && m_useownsymbols == OWN_SYMBOLS) {
+      CString cs_symbols;
+      m_symbols.GetWindowText(cs_symbols);
+      if (m_pwusesymbols == TRUE && !cs_symbols.IsEmpty()) {
         M_iownsymbols() = OWN_SYMBOLS;
-        CString cs_symbols;
-        m_symbols.GetWindowText(cs_symbols);
         M_symbols() = CSecString(cs_symbols);
       } else {
-        m_useownsymbols = DEFAULT_SYMBOLS;
         M_iownsymbols() = DEFAULT_SYMBOLS;
       }
       break;
@@ -913,17 +807,6 @@ void CAddEdit_PasswordPolicy::SetVariablesFromPolicy()
   m_pwlowerminlength = M_pwp().lowerminlength;
   m_pwsymbolminlength = M_pwp().symbolminlength;
   m_pwupperminlength = M_pwp().upperminlength;
-
-  if (m_pwuselowercase == TRUE && m_pwlowerminlength == 0)
-    m_pwlowerminlength = 1;
-  if (m_pwuseuppercase == TRUE && m_pwupperminlength == 0)
-    m_pwupperminlength = 1;
-  if (m_pwusedigits == TRUE && m_pwdigitminlength == 0)
-    m_pwdigitminlength = 1;
-  if (m_pwusesymbols == TRUE && m_pwsymbolminlength == 0)
-    m_pwsymbolminlength = 1;
-
-  m_useownsymbols = M_iownsymbols();
 }
 
 void CAddEdit_PasswordPolicy::DisablePolicy()
@@ -991,3 +874,16 @@ void CAddEdit_PasswordPolicy::OnNamesComboChanged()
   SetVariablesFromPolicy();
   SetPolicyControls();
 }
+
+void CAddEdit_PasswordPolicy::OnSymbolReset()
+{
+  std::wstring symbols;
+  if (m_pweasyvision)
+    symbols = CPasswordCharPool::GetEasyVisionSymbols();
+  else if (m_pwmakepronounceable)
+    symbols = CPasswordCharPool::GetPronounceableSymbols();
+  else
+    symbols = CPasswordCharPool::GetDefaultSymbols();
+  m_symbols.SetValidSym(symbols);
+  m_symbols.SetWindowText(symbols.c_str());
+}
