@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -11,11 +11,11 @@
 
 #include "stdafx.h"
 
-#include "Windowsdefs.h"
 #include "GeneralMsgBox.h"
 #include "DboxMain.h"
 #include "CompareResultsDlg.h"
 #include "ShowCompareDlg.h"
+#include "Fonts.h"
 
 #include "core/core.h"
 #include "core/PWScore.h"
@@ -73,6 +73,7 @@ CCompareResultsDlg::CCompareResultsDlg(CWnd* pParent,
                                        CompareData &Conflicts, CompareData &Identical,
                                        CItemData::FieldBits &bsFields,
                                        PWScore *pcore0, PWScore *pcore1,
+                                       CString &csProtect, CString &csAttachment,
                                        CReport *pRpt)
   : CPWResizeDialog(CCompareResultsDlg::IDD, pParent),
   m_OnlyInCurrent(OnlyInCurrent), m_OnlyInComp(OnlyInComp),
@@ -81,7 +82,8 @@ CCompareResultsDlg::CCompareResultsDlg(CWnd* pParent,
   m_pRpt(pRpt), m_bSortAscending(true), m_iSortedColumn(0),
   m_OriginalDBChanged(false),
   m_bTreatWhiteSpaceasEmpty(false),
-  m_ShowIdenticalEntries(BST_UNCHECKED)
+  m_ShowIdenticalEntries(BST_UNCHECKED),
+  m_csProtect(csProtect), m_csAttachment(csAttachment)
 {
 }
 
@@ -111,13 +113,16 @@ void CCompareResultsDlg::DoDataExchange(CDataExchange* pDX)
 BEGIN_MESSAGE_MAP(CCompareResultsDlg, CPWResizeDialog)
   ON_WM_SIZE()
   ON_WM_INITMENUPOPUP()
+  
   ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDOK, OnOK)
   ON_BN_CLICKED(IDC_SHOW_IDENTICAL_ENTRIES, OnShowIdenticalEntries)
+  
   ON_NOTIFY(NM_RCLICK, IDC_RESULTLIST, OnItemRightClick)
   ON_NOTIFY(NM_DBLCLK, IDC_RESULTLIST, OnItemDoubleClick)
   ON_NOTIFY(LVN_ITEMCHANGING, IDC_RESULTLIST, OnItemChanging)
   ON_NOTIFY(HDN_ITEMCLICK, IDC_RESULTLISTHDR, OnColumnClick)
+  
   ON_COMMAND(ID_MENUITEM_COMPVIEWEDIT, OnCompareViewEdit)
   ON_COMMAND(ID_MENUITEM_SYNCHRONIZE, OnCompareSynchronize)
   ON_COMMAND(ID_MENUITEM_COPY_TO_ORIGINAL, OnCompareCopyToOriginalDB)
@@ -153,6 +158,8 @@ BOOL CCompareResultsDlg::OnInitDialog()
   DWORD dwExtendedStyle = m_LCResults.GetExtendedStyle();
   dwExtendedStyle |= (LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
   m_LCResults.SetExtendedStyle(dwExtendedStyle);
+  
+  m_LCResults.UpdateRowHeight(false);
 
   CString cs_header;
   int i;
@@ -166,8 +173,7 @@ BOOL CCompareResultsDlg::OnInitDialog()
     if (m_bsFields.test(OptCols[i].ft)) {
       cs_header.LoadString(OptCols[i].ids);
       // Add on the end
-      int icol = m_LCResults.InsertColumn(LAST, cs_header, LVCFMT_CENTER);
-      ASSERT(icol != -1);
+      VERIFY(m_LCResults.InsertColumn(LAST, cs_header, LVCFMT_CENTER) != -1);
     }
   }
 
@@ -220,7 +226,7 @@ BOOL CCompareResultsDlg::OnInitDialog()
 
   WriteReportData();
   UpdateStatusBar();
-  return FALSE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 void CCompareResultsDlg::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
@@ -251,10 +257,10 @@ void CCompareResultsDlg::AddCompareEntries(const bool bAddIdentical)
 
       StringX sxTitle = st_data.title;
       if (st_data.bIsProtected0)
-        sxTitle += L" #";
+        sxTitle += m_csProtect;
 
       if (st_data.bHasAttachment0)
-        sxTitle += L" +";
+        sxTitle += m_csAttachment;
       
       m_LCResults.SetItemText(iItem, TITLE, sxTitle.c_str());
 
@@ -281,10 +287,10 @@ void CCompareResultsDlg::AddCompareEntries(const bool bAddIdentical)
       m_LCResults.SetItemText(iItem, GROUP, st_data.group.c_str());
       StringX sxTitle = st_data.title;
       if (st_data.bIsProtected0)
-        sxTitle += L" #";
+        sxTitle += m_csProtect;
 
       if (st_data.bHasAttachment0)
-        sxTitle += L" +";
+        sxTitle += m_csAttachment;
 
       m_LCResults.SetItemText(iItem, TITLE, sxTitle.c_str());
       m_LCResults.SetItemText(iItem, USER, st_data.user.c_str());
@@ -336,10 +342,10 @@ void CCompareResultsDlg::AddCompareEntries(const bool bAddIdentical)
       m_LCResults.SetItemText(iItem, GROUP, st_data.group.c_str());
       StringX sxTitle = st_data.title;
       if (st_data.bIsProtected0)
-        sxTitle += L" #";
+        sxTitle += m_csProtect;
 
       if (st_data.bHasAttachment0)
-        sxTitle += L" +";
+        sxTitle += m_csAttachment;
 
       m_LCResults.SetItemText(iItem, TITLE, sxTitle.c_str());
       m_LCResults.SetItemText(iItem, USER, st_data.user.c_str());
@@ -953,8 +959,7 @@ void CCompareResultsDlg::OnItemRightClick(NMHDR *pNMHDR, LRESULT *pLResult)
       minfo.cbSize = sizeof(MENUINFO);
       minfo.fMask = MIM_MENUDATA;
       minfo.dwMenuData = ipopup;
-      BOOL brc = menu.SetMenuInfo(&minfo);
-      ASSERT(brc != 0);
+      VERIFY(menu.SetMenuInfo(&minfo));
 
       CMenu *pPopup = menu.GetSubMenu(0);
       ASSERT(pPopup != NULL);
@@ -1008,8 +1013,7 @@ void CCompareResultsDlg::OnItemRightClick(NMHDR *pNMHDR, LRESULT *pLResult)
     minfo.cbSize = sizeof(MENUINFO);
     minfo.fMask = MIM_MENUDATA;
     minfo.dwMenuData = ipopup;
-    BOOL brc = menu.SetMenuInfo(&minfo);
-    ASSERT(brc != 0);
+    VERIFY(menu.SetMenuInfo(&minfo));
 
     CMenu *pPopup = menu.GetSubMenu(0);
     ASSERT(pPopup != NULL);
@@ -1142,11 +1146,6 @@ void CCompareResultsDlg::OnColumnClick(NMHDR *pNotifyStruct, LRESULT *pLResult)
     pst_data->listindex = i;
   }
 
-#if (WINVER < 0x0501)  // These are already defined for WinXP and later
-#define HDF_SORTUP   0x0400
-#define HDF_SORTDOWN 0x0200
-#endif
-
   HDITEM hdi;
   hdi.mask = HDI_FORMAT;
 
@@ -1443,22 +1442,24 @@ bool CCompareResultsDlg::CompareEntries(st_CompareData *pst_data)
 
 // Compare CListCtrl
 
-CCPListCtrl::CCPListCtrl()
+CCPListCtrlX::CCPListCtrlX()
   : m_row(-1), m_column(-1)
 {
 }
 
-CCPListCtrl::~CCPListCtrl()
+CCPListCtrlX::~CCPListCtrlX()
 {
 }
 
-BEGIN_MESSAGE_MAP(CCPListCtrl, CListCtrl)
-  //{{AFX_MSG_MAP(CCPListCtrl)
+BEGIN_MESSAGE_MAP(CCPListCtrlX, CListCtrl)
+  //{{AFX_MSG_MAP(CCPListCtrlX)
+  ON_WM_MEASUREITEM_REFLECT()
+  
   ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, OnCustomDraw)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
-BOOL CCPListCtrl::PreTranslateMessage(MSG *pMsg)
+BOOL CCPListCtrlX::PreTranslateMessage(MSG *pMsg)
 {
   if (pMsg->message == WM_LBUTTONDOWN) {
     // Get cell position for CustomDraw
@@ -1482,9 +1483,23 @@ BOOL CCPListCtrl::PreTranslateMessage(MSG *pMsg)
     }
   }
   return CListCtrl::PreTranslateMessage(pMsg);
-} 
+}
 
-bool CCPListCtrl::IsSelected(DWORD_PTR iRow)
+void CCPListCtrlX::MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct)
+{
+  if (!Fonts::GetInstance())
+    return;
+
+  int padding = 4;
+  if (GetExtendedStyle() & LVS_EX_GRIDLINES)
+    padding += 2;
+
+  lpMeasureItemStruct->itemHeight = Fonts::GetInstance()->CalcHeight() + padding;
+  //Remove LVS_OWNERDRAWFIXED style to apply default DrawItem
+  ModifyStyle(LVS_OWNERDRAWFIXED, 0);
+}
+
+bool CCPListCtrlX::IsSelected(DWORD_PTR iRow)
 {
   POSITION pos = GetFirstSelectedItemPosition();
 
@@ -1496,7 +1511,7 @@ bool CCPListCtrl::IsSelected(DWORD_PTR iRow)
   return false;
 }
 
-void CCPListCtrl::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
+void CCPListCtrlX::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
 {
   NMLVCUSTOMDRAW *pLVCD = reinterpret_cast<NMLVCUSTOMDRAW *>(pNotifyStruct);
 
@@ -1504,14 +1519,22 @@ void CCPListCtrl::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
 
   static COLORREF crWindowText, cfNormalTextBkgrd, crSelectedText, crSelectedBkgrd;
   bool bIsSelected = IsSelected(pLVCD->nmcd.dwItemSpec);
-  
+
+  static bool bchanged_subitem_font(false);
+  static CDC *pDC = NULL;
+  static CFont *pAddEditFont = NULL;
+
   switch (pLVCD->nmcd.dwDrawStage) {
     case CDDS_PREPAINT:
       // PrePaint
+      bchanged_subitem_font = false;
       crWindowText = GetTextColor();
       cfNormalTextBkgrd = GetTextBkColor();
       crSelectedText = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
       crSelectedBkgrd = ::GetSysColor(COLOR_HIGHLIGHT);
+      
+      pDC = CDC::FromHandle(pLVCD->nmcd.hdc);
+      pAddEditFont = Fonts::GetInstance()->GetAddEditFont();
       *pLResult = CDRF_NOTIFYITEMDRAW;
       break;
 
@@ -1521,9 +1544,10 @@ void CCPListCtrl::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
         pLVCD->clrText = crWindowText;
         pLVCD->clrTextBk = cfNormalTextBkgrd;
         pLVCD->nmcd.uItemState &= ~CDIS_SELECTED;
-        *pLResult |= CDRF_NEWFONT;
       }
-      *pLResult |= CDRF_NOTIFYSUBITEMDRAW;
+      pDC->SelectObject(pAddEditFont);
+      bchanged_subitem_font = true;
+      *pLResult |= (CDRF_NOTIFYSUBITEMDRAW | CDRF_NEWFONT);
       break;
 
     case CDDS_ITEMPREPAINT | CDDS_SUBITEM:
@@ -1540,6 +1564,15 @@ void CCPListCtrl::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
       *pLResult |= (CDRF_NOTIFYPOSTPAINT | CDRF_NEWFONT);
       break;
 
+    case CDDS_ITEMPOSTPAINT | CDDS_SUBITEM:
+      // Sub-item PostPaint - restore old font if any
+      if (bchanged_subitem_font) {
+        bchanged_subitem_font = false;
+        pDC->SelectObject(pAddEditFont);
+        *pLResult |= CDRF_NEWFONT;
+      }
+      break;
+
     /*
     case CDDS_PREERASE:
     case CDDS_POSTERASE:
@@ -1553,3 +1586,26 @@ void CCPListCtrl::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
       break;
   }
 }
+
+void CCPListCtrlX::UpdateRowHeight(bool bInvalidate) {
+  // We need to change WINDOWPOS to trigger MeasureItem 
+  // http://www.codeproject.com/Articles/1401/Changing-Row-Height-in-an-owner-drawn-Control
+  CRect rc;
+  GetWindowRect(&rc);
+  WINDOWPOS wp;
+  wp.hwnd = m_hWnd;
+  wp.cx = rc.Width();
+  wp.cy = rc.Height();
+  wp.flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER;
+
+  //Add LVS_OWNERDRAWFIXED style for generating MeasureItem event
+  ModifyStyle(0, LVS_OWNERDRAWFIXED);
+
+  SendMessage(WM_WINDOWPOSCHANGED, 0, (LPARAM)&wp);
+  if (bInvalidate) {
+    Invalidate();
+    int idx = GetTopIndex();
+    if (idx >= 0)
+      EnsureVisible(idx, FALSE);
+  }
+}
