@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -30,6 +30,7 @@
 #include "MenuShortcuts.h"
 #include "AdvancedDlg.h"
 #include "FontsDialog.h"
+#include "SystemTray.h"
 
 #include "core/UIinterface.h"
 #include "core/PWScore.h"
@@ -48,28 +49,35 @@
 #include <list>
 #include <stack>
 
-#if (WINVER < 0x0501)  // These are already defined for WinXP and later
-#define HDF_SORTUP             0x0400
-#define HDF_SORTDOWN           0x0200
-
-#define WM_WTSSESSION_CHANGE   0x02B1
-
-#define WTS_CONSOLE_CONNECT                0x1
-#define WTS_CONSOLE_DISCONNECT             0x2
-#define WTS_REMOTE_CONNECT                 0x3
-#define WTS_REMOTE_DISCONNECT              0x4
-#define WTS_SESSION_LOGON                  0x5
-#define WTS_SESSION_LOGOFF                 0x6
-#define WTS_SESSION_LOCK                   0x7
-#define WTS_SESSION_UNLOCK                 0x8
-#define WTS_SESSION_REMOTE_CONTROL         0x9
-
-#endif  /* WINVER < 0x0501 */
-
 // For ShutdownBlockReasonCreate & ShutdownBlockReasonDestroy
 typedef BOOL (WINAPI *PSBR_CREATE) (HWND, LPCWSTR);
 typedef BOOL (WINAPI *PSBR_DESTROY) (HWND);
 
+// Entry to GUI mapping
+// Following used to keep track of display vs data
+// stored as opaque data in m_MapEntryToGUI
+// Exposed here because PWTreeCtrl needs to update it after drag&drop
+struct DisplayInfo {
+  int list_index;
+  HTREEITEM tree_item;
+
+  DisplayInfo() :list_index(-1), tree_item(0) {}
+  ~DisplayInfo() {}
+
+  DisplayInfo(const DisplayInfo &that)
+    : list_index(that.list_index), tree_item(that.tree_item)
+  {}
+
+  DisplayInfo &operator=(const DisplayInfo &that)
+  {
+    if (this != &that) {
+      list_index = that.list_index;
+      tree_item = that.tree_item;
+    }
+    return *this;
+  }
+};
+
 enum SearchDirection {FIND_UP = -1, FIND_DOWN = 1};
 
 // List of all Popup menus in the Main Menu
@@ -113,6 +121,8 @@ public:
   enum SaveType {ST_INVALID = -1, ST_NORMALEXIT = 0, ST_SAVEIMMEDIATELY,
                  ST_ENDSESSIONEXIT, ST_WTSLOGOFFEXIT, ST_FAILSAFESAVE};
 
+  enum DBSTATE { LOCKED, UNLOCKED, CLOSED };
+
   // Find entry by title and user name, exact match
   ItemListIter Find(const StringX &a_group,
                     const StringX &a_title, const StringX &a_user)
@@ -134,14 +144,17 @@ public:
 
   // FindAll is used by CPWFindToolBar, returns # of finds.
   size_t FindAll(const CString &str, BOOL CaseSensitive,
-                 std::vector<int> &indices);
+                 std::vector<int> &vIndices, std::vector<pws_os::CUUID> &vFoundUUIDs);
   size_t FindAll(const CString &str, BOOL CaseSensitive,
-                 std::vector<int> &indices,
+                 std::vector<int> &vIndices,
+                 std::vector<pws_os::CUUID> &vFoundUUIDs,
                  const CItemData::FieldBits &bsFields, 
                  const CItemAtt::AttFieldBits &bsAttFields, 
                  const bool &subgroup_set, const std::wstring &subgroup_name,
                  const int subgroup_object, const int subgroup_function);
 
+  void SetFilterFindEntries(std::vector<pws_os::CUUID> *pvFoundUUIDs);
+
   // Used by ListCtrl KeyDown
   bool IsImageVisible() const {return m_bImageInLV;}
 
@@ -157,7 +170,7 @@ public:
 
   // For InsertItemIntoGUITreeList and RefreshViews (mainly when refreshing views)
   // Note: iBothViews = iListOnly + iTreeOnly
-  enum ViewType {LISTONLY = 1, TREEONLY = 2, BOTHVIEWS = 3};
+  enum ViewType { NONE = 0, LISTONLY = 1, TREEONLY = 2, BOTHVIEWS = 3 };
 
   void RefreshViews(const ViewType iView = BOTHVIEWS);
 
@@ -205,7 +218,6 @@ public:
   void UpdateToolBarROStatus(const bool bIsRO);
   void UpdateToolBarForSelectedItem(const CItemData *pci);
   void SetToolBarPositions();
-  void InvalidateSearch() {m_FindToolBar.InvalidateSearch();}
   void ResumeOnDBNotification() {m_core.ResumeOnDBNotification();}
   void SuspendOnDBNotification() {m_core.SuspendOnDBNotification();}
   bool GetDBNotificationState() {return m_core.GetDBNotificationState();}
@@ -225,7 +237,7 @@ public:
   bool ExitRequested() const {return m_inExit;}
   void AutoResizeColumns();
   void ResetIdleLockCounter(UINT event = WM_SIZE); // default arg always resets
-  bool ClearClipboardData() {return m_clipboard.ClearData();}
+  bool ClearClipboardData() {return m_clipboard.ClearCBData();}
   bool SetClipboardData(const StringX &data)
   {return m_clipboard.SetData(data.c_str());}
   void AddDDEntries(CDDObList &in_oblist, const StringX &DropGroup,
@@ -256,7 +268,7 @@ public:
   HICON GetEntryIcon(const int nImage) const;
   void SetEntryImage(const int &index, const int nImage, const bool bOneEntry = false);
   void SetEntryImage(HTREEITEM &ti, const int nImage, const bool bOneEntry = false);
-  void UpdateEntryImages(const CItemData &ci);
+  void UpdateEntryImages(const CItemData &ci, bool bAllowFail = false);
 
   void RefreshImages();
   void CreateShortcutEntry(CItemData *pci, const StringX &cs_group,
@@ -325,13 +337,17 @@ public:
 
   std::wstring DoMerge(PWScore *pothercore,
                   const bool bAdvanced, CReport *prpt, bool *pbCancel);
+  
   bool DoCompare(PWScore *pothercore,
                  const bool bAdvanced, CReport *prpt, bool *pbCancel);
+  
   void DoSynchronize(PWScore *pothercore,
                      const bool bAdvanced, int &numUpdated, CReport *prpt, bool *pbCancel);
+  
   int DoExportText(const StringX &sx_Filename, const UINT nID,
                    const wchar_t &delimiter, const bool bAdvanced,
                    int &numExported, CReport *prpt);
+  
   int DoExportXML(const StringX &sx_Filename, const UINT nID,
                   const wchar_t &delimiter, const bool bAdvanced,
                   int &numExported, CReport *prpt);
@@ -348,7 +364,7 @@ public:
   {return m_core.TestSelection(bAdvanced, subgroup_name,
                                subgroup_object, subgroup_function, pOIL);}
 
-  void MakeOrderedItemList(OrderedItemList &OIL, HTREEITEM hItem = NULL);
+  std::vector<pws_os::CUUID> MakeOrderedItemList(OrderedItemList &OIL, HTREEITEM hItem = NULL);
   bool MakeMatchingGTUSet(GTUSet &setGTU, const StringX &sxPolicyName) const
   {return m_core.InitialiseGTU(setGTU, sxPolicyName);}
   CItemData *getSelectedItem();
@@ -364,6 +380,15 @@ public:
   bool IsInRename() const {return m_bInRename;}
   bool IsInAddGroup() const {return m_bInAddGroup;}
   void ResetInAddGroup() {m_bInAddGroup = false;}
+  void SetRenameGroups(const StringX sxNewPath)
+  { m_sxNewPath = sxNewPath; }
+
+  int GetDBIndex() { return m_iDBIndex; }
+  COLORREF GetLockedIndexColour() { return m_DBLockedIndexColour; }
+  COLORREF GetUnlockedIndexColour() { return m_DBUnlockedIndexColour; }
+
+  DBSTATE GetSystemTrayState() const { return m_TrayLockedState; }
+  BOOL IsIconVisible() const { return m_pTrayIcon->Visible(); }
 
   //{{AFX_DATA(DboxMain)
   enum { IDD = IDD_PASSWORDSAFE_DIALOG };
@@ -390,8 +415,11 @@ public:
   // Used in Add & Edit & PasswordPolicyDlg
   PWPolicy m_pwp;
 
+  // Get link between entry and GUI
+  DisplayInfo *GetEntryGUIInfo(const CItemData &ci, bool bAllowFail = false);
+
   // Mapping Group to Tree Item to save searching all the time!
-  // Be nice to have a bimap implementation
+  // Be nice to have a bitmap implementation
   std::map<StringX, HTREEITEM> m_mapGroupToTreeItem;
   std::map<HTREEITEM, StringX> m_mapTreeItemToGroup;
   void GetAllGroups(std::vector<std::wstring> &vGroups) const;
@@ -401,9 +429,9 @@ public:
   bool CheckPreTranslateRename(MSG* pMsg);
   bool CheckPreTranslateAutoType(MSG* pMsg);
 
-  void SetSetup() {m_bSetup = true;}                     // called by app when '--setup' passed
-  void NoValidation() {m_bNoValidation = true;}          // called by app when '--novalidate' passed
-  void AllowCompareEntries() {m_bCompareEntries = true;} // called by app when '--cetreeview' passed
+  void SetSetup() {m_bSetup = true;}                     // called when '--setup' passed
+  void NoValidation() {m_bNoValidation = true;}          // called when '--novalidate' passed
+  void AllowCompareEntries() {m_bCompareEntries = true;} // called when '--cetreeview' passed
 
   // Needed public function for ComapreResultsDialog
   void CPRInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
@@ -415,22 +443,30 @@ public:
   const unsigned int GetMenuShortcut(const unsigned short int &siVirtKey,
                                      const unsigned char &cModifier, StringX &sxMenuItemName);
   
-  void ChangeMode(bool promptUser); // r-o <-> r/w
+  bool ChangeMode(bool promptUser); // r-o <-> r/w
 
   // If we have processed it returns 0 else 1
-  BOOL ProcessEntryShortcut(WORD &wVirtualKeyCode, WORD &wModifiers);
+  BOOL ProcessEntryShortcut(WORD &wVirtualKeyCode, WORD &wWinModifiers);
   bool IsWorkstationLocked() const;
   void BlockLogoffShutdown(const bool bChanged);
 
   std::set<StringX> GetAllMediaTypes() const
   {return m_core.GetAllMediaTypes();}
 
+  // For latered Windows
+  PSLWA GetSetLayeredWindowAttributes() { return m_pfcnSetLayeredWindowAttributes; }
+  bool GetInitialTransparencyState() { return m_bOnStartupTransparancyEnabled; }
+  bool SetLayered(CWnd *pWnd, const int value = -1);
+
  protected:
+   friend class CSetDBID;  // To access icon creation etc.
+
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(DboxMain)
+   virtual BOOL PreTranslateMessage(MSG *pMsg);
    virtual BOOL OnInitDialog();
    virtual void OnCancel();
-   virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
+   virtual void DoDataExchange(CDataExchange *pDX);  // DDX/DDV support
    // override following to reset idle timeout on any event
    virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
    //}}AFX_VIRTUAL
@@ -441,10 +477,16 @@ public:
   // used to speed up the resizable dialog so OnSize/SIZE_RESTORED isn't called
   bool m_bSizing;
   bool m_bIsRestoring;
+
+  // Is DB open
   bool m_bOpen;
+
+  // Other bool
   bool m_bInRestoreWindowsData;
   bool m_bUserDeclinedSave;
   bool m_bRestoredDBUnsaved;
+  bool m_bSuspendGUIUpdates;
+  ViewType m_iNeedRefresh;
 
   bool m_bSetup;          // invoked with '--setup'?
   bool m_bNoValidation;   // invoked with '--novalidate'?
@@ -477,8 +519,9 @@ public:
   int m_treatwhitespaceasempty;
 
   HTREEITEM m_LastFoundTreeItem;
-  int m_LastFoundListItem;
+  int m_LastFoundListItem, m_iCurrentItemFound;
   bool m_bBoldItem;
+  bool m_bFindToolBarVisibleAtLock;
 
   WCHAR *m_pwchTip;
   
@@ -497,13 +540,20 @@ public:
   // For UPDATE_UI
   int OnUpdateMenuToolbar(const UINT nID);
   int OnUpdateViewReports(const int nID);
-  void OnUpdateMRU(CCmdUI* pCmdUI);
+  void OnUpdateMRU(CCmdUI *pCmdUI);
 
   void ConfigureSystemMenu();
 
-  // 'STATE' also defined in ThisMfcApp.h - ensure identical
-  enum STATE { LOCKED, UNLOCKED, CLOSED };
-  void UpdateSystemTray(const STATE s);
+  void UpdateSystemTray(const DBSTATE s);
+  BOOL SetTooltipText(LPCWSTR ttt) { return m_pTrayIcon->SetTooltipText(ttt); }
+  void ShowIcon() { m_pTrayIcon->ShowIcon(); }
+  void HideIcon() { m_pTrayIcon->HideIcon(); }
+
+  void SetSystemTrayState(DBSTATE s);
+  void SetSystemTrayTarget(CWnd *pWnd) { m_pTrayIcon->SetTarget(pWnd); }
+
+  HICON CreateIcon(const HICON &hIcon, const int &iIndex,
+                   const COLORREF clrText = RGB(255, 255, 0));
 
   LRESULT OnHotKey(WPARAM wParam, LPARAM lParam);
   LRESULT OnCCToHdrDragComplete(WPARAM wParam, LPARAM lParam);
@@ -527,10 +577,8 @@ public:
   LRESULT OnApplyEditChanges(WPARAM wParam, LPARAM lParam);
   LRESULT OnDroppedFile(WPARAM wParam, LPARAM lParam);
 
-  BOOL PreTranslateMessage(MSG* pMsg);
-
   void UpdateAlwaysOnTop();
-  void ClearData(const bool bClearMRE = true);
+  void ClearAppData(const bool bClearMRE = true);
   int NewFile(StringX &filename);
 
   void SetListView();
@@ -540,7 +588,7 @@ public:
   void UpdateMenuAndToolBar(const bool bOpen);
   void SortListView();
 
-  //Version of message functions with return values
+  // Version of message functions with return values
   int Save(const SaveType savetype = DboxMain::ST_INVALID);
   int SaveAs();
   int Open(const UINT uiTitle = IDS_CHOOSEDATABASE);
@@ -567,6 +615,7 @@ public:
   void SaveGUIStatus();
   void RestoreGUIStatus();
 
+  void SetupUserFonts();
   void ChangeFont(const CFontsDialog::FontType iType);
 
   // Generated message map functions
@@ -574,7 +623,6 @@ public:
   afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
   afx_msg void OnTrayLockUnLock();
   afx_msg void OnTrayClearRecentEntries();
-  afx_msg void OnUpdateTrayClearRecentEntries(CCmdUI *pCmdUI);
   afx_msg void OnTrayCopyUsername(UINT nID);
   afx_msg void OnUpdateTrayCopyUsername(CCmdUI *pCmdUI);
   afx_msg void OnTrayCopyPassword(UINT nID);
@@ -597,6 +645,8 @@ public:
   afx_msg void OnUpdateTraySendEmail(CCmdUI *pCmdUI);
   afx_msg void OnTraySelect(UINT nID);
   afx_msg void OnUpdateTraySelect(CCmdUI *pCmdUI);
+  afx_msg void OnGotoDependant(UINT nID);
+  afx_msg void OnUpdateGotoDependant(CCmdUI *pCmdUI);
 
   afx_msg LRESULT OnAreYouMe(WPARAM, LPARAM);
   afx_msg LRESULT OnWH_SHELL_CallBack(WPARAM wParam, LPARAM lParam);
@@ -657,6 +707,7 @@ public:
   afx_msg void OnManagePasswordPolicies();
   afx_msg void OnGeneratePassword();
   afx_msg void OnYubikey();
+  afx_msg void OnSetDBID();
   afx_msg void OnSave();
   afx_msg void OnAdd();
   afx_msg void OnAddGroup();
@@ -669,7 +720,6 @@ public:
   afx_msg void OnShowHideDragbar();
   afx_msg void OnOldToolbar();
   afx_msg void OnNewToolbar();
-  afx_msg void OnShowFindToolbar();
   afx_msg void OnExpandAll();
   afx_msg void OnCollapseAll();
   afx_msg void OnChangeTreeFont();
@@ -680,12 +730,14 @@ public:
   afx_msg void OnViewReportsByID(UINT nID);  // From View->Reports menu
   afx_msg void OnViewReports();
   afx_msg void OnManageFilters(); // From Toolbar button
+  afx_msg void OnExportFilteredDB();
   afx_msg void OnCancelFilter();
   afx_msg void OnApplyFilter();
   afx_msg void OnSetFilter();
   afx_msg void OnRefreshWindow();
   afx_msg void OnShowUnsavedEntries();
   afx_msg void OnShowExpireList();
+  afx_msg void OnShowFoundEntries();
   afx_msg void OnMinimize();
   afx_msg void OnRestore();
   afx_msg void OnTimer(UINT_PTR nIDEvent);
@@ -727,7 +779,8 @@ public:
   afx_msg void OnToolBarFindAdvanced();
   afx_msg void OnToolBarFindReport();
   afx_msg void OnToolBarClearFind();
-  afx_msg void OnHideFindToolBar();
+  afx_msg void OnShowFindToolbar();
+  afx_msg void OnHideFindToolbar();
 
   afx_msg void OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct);
   afx_msg void OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct);
@@ -745,9 +798,12 @@ private:
   virtual void DatabaseModified(bool bChanged);
   virtual void UpdateGUI(UpdateGUICommand::GUI_Action ga,
                          const pws_os::CUUID &entry_uuid,
-                         CItemData::FieldType ft, bool bUpdateGUI);
-  virtual void GUISetupDisplayInfo(CItemData &ci);
-  virtual void GUIRefreshEntry(const CItemData &ci);
+                         CItemData::FieldType ft);
+  // Version for groups
+  virtual void UpdateGUI(UpdateGUICommand::GUI_Action ga,
+                         const std::vector<StringX> &vGroups);
+  
+  virtual void GUIRefreshEntry(const CItemData &ci, bool bAllowFail = false);
   virtual void UpdateWizard(const std::wstring &s);
 
   static int CALLBACK CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
@@ -760,6 +816,14 @@ private:
 
   CPasskeyEntry *m_pPasskeyEntryDlg;
 
+  CSystemTray *m_pTrayIcon; // DboxMain needs to be constructed first
+  DBSTATE m_TrayLockedState;
+
+  HICON m_LockedIcon;
+  HICON m_UnLockedIcon;
+  HICON m_ClosedIcon;
+  HICON m_IndexIcon;
+
   bool m_IsStartSilent;
   bool m_IsStartClosed;
   bool m_bStartHiddenAndMinimized;
@@ -769,6 +833,7 @@ private:
   bool m_bInRefresh, m_bInRestoreWindows;
   bool m_bDBInitiallyRO;
   bool m_bViaDCA;
+  bool m_bFindBarShown;
   int m_iDateTimeFieldWidth;
   int m_nColumns;
   int m_nColumnIndexByOrder[CItem::LAST_DATA];
@@ -778,6 +843,7 @@ private:
   int m_nColumnHeaderWidthByType[CItem::LAST_DATA];
   int m_nSaveColumnHeaderWidthByType[CItem::LAST_DATA];
   int m_iheadermaxwidth;
+  int m_iListHBarPos, m_iTreeHBarPos;
 
   pws_os::CUUID m_LUUIDSelectedAtMinimize; // to restore List entry selection upon un-minimize
   pws_os::CUUID m_TUUIDSelectedAtMinimize; // to restore Tree entry selection upon un-minimize
@@ -785,6 +851,19 @@ private:
   pws_os::CUUID m_LUUIDVisibleAtMinimize;  // to restore List entry position  upon un-minimize
   pws_os::CUUID m_TUUIDVisibleAtMinimize;  // to restore Tree entry position  upon un-minimize
   StringX m_sxVisibleGroup;                // to restore Tree group position  upon un-minimize
+ 
+  // Here lies the mapping between an entry and its place on the GUI (Tree/List views)
+  std::map<pws_os::CUUID, DisplayInfo, std::less<pws_os::CUUID> > m_MapEntryToGUI;
+
+  // Mapping between visible dependants and their base (might not be visible if filter active)
+  std::vector<int> m_vGotoDependants;
+  
+  // Set link between entry and GUI
+  void SetEntryGUIInfo(const CItemData &ci, const DisplayInfo &di)
+  { m_MapEntryToGUI[ci.GetUUID()] = di; } // often used to update, so map::insert() is inappropriate
+
+  // Used in SaveGUIStatus to remember position during rename group - set by CPWTreeCtrl
+  StringX m_sxNewPath;
 
   StringX m_sxOriginalGroup;                 // Needed when doing recursive deletions of groups
 
@@ -835,13 +914,15 @@ private:
   void RestoreWindows(); // extended ShowWindow(SW_RESTORE), sort of
   void CancelPendingPasswordDialog();
 
-  void RemoveFromGUI(CItemData &ci, bool bUpdateGUI);
+  void RemoveFromGUI(CItemData &ci);
   void AddToGUI(CItemData &ci);
   void RefreshEntryFieldInGUI(CItemData &ci, CItemData::FieldType ft);
   void RefreshEntryPasswordInGUI(CItemData &ci);
   void RebuildGUI(const ViewType iView = BOTHVIEWS);
-  void UpdateEntryinGUI(CItemData &ci);
+  void UpdateEntryInGUI(CItemData &ci);
+  void UpdateGroupsInGUI(const std::vector<StringX> &vGroups);
   StringX GetListViewItemText(CItemData &ci, const int &icolumn);
+  void DoCommand(Command *pcmd = NULL, PWScore *pcore = NULL, const bool bUndo = true);
   
   static const struct UICommandTableEntry {
     UINT ID;
@@ -861,7 +942,7 @@ private:
   CInfoDisplay *m_pNotesDisplay;
 
   // Filters
-  bool m_bFilterActive, m_bUnsavedDisplayed, m_bExpireDisplayed;
+  bool m_bFilterActive, m_bUnsavedDisplayed, m_bExpireDisplayed, m_bFindFilterDisplayed;
   // Current filter
   st_filters &CurrentFilter() {return m_FilterManager.m_currentfilter;}
 
@@ -900,6 +981,9 @@ private:
   PSBR_CREATE m_pfcnShutdownBlockReasonCreate;
   PSBR_DESTROY m_pfcnShutdownBlockReasonDestroy;
 
+  // For Layered Windows
+  PSLWA m_pfcnSetLayeredWindowAttributes;
+
   // Delete/Rename/AutoType Shortcuts
   WPARAM m_wpDeleteMsg, m_wpDeleteKey;
   WPARAM m_wpRenameMsg, m_wpRenameKey;
@@ -907,6 +991,7 @@ private:
   bool m_bDeleteCtrl, m_bDeleteShift;
   bool m_bRenameCtrl, m_bRenameShift;
   bool m_bAutotypeCtrl, m_bAutotypeShift;
+  bool m_bOnStartupTransparancyEnabled;
 
   // Do Autotype
   bool m_bInAT;
@@ -940,6 +1025,11 @@ private:
   int m_ilastaction;  // Last action
   void SetDragbarToolTips();
 
+  // Database index on Tray icon
+  int m_iDBIndex;
+  COLORREF m_DBLockedIndexColour, m_DBUnlockedIndexColour;
+  HANDLE m_hMutexDBIndex;
+
   // The following is for saving information over an execute/undo/redo
   // Might need to add more e.g. if filter is active and which one?
   struct st_SaveGUIInfo {
@@ -977,28 +1067,16 @@ private:
   std::stack<st_SaveGUIInfo> m_stkSaveGUIInfo;
 };
 
-// Following used to keep track of display vs data
-// stored as opaque data in CItemData.{Get,Set}DisplayInfo()
-// Exposed here because PWTreeCtrl needs to update it after drag&drop
-struct DisplayInfo : public DisplayInfoBase {
-  int list_index;
-  HTREEITEM tree_item;
-
- DisplayInfo() :list_index(-1), tree_item(0) {}
-  virtual ~DisplayInfo() {}
-  virtual DisplayInfo *clone() const // virtual c'tor idiom
-  { return new DisplayInfo(*this); }
- 
-  DisplayInfo(const DisplayInfo &that)
-  : list_index(that.list_index), tree_item(that.tree_item)
-  {}
-
-  DisplayInfo &operator=(const DisplayInfo &that)
-  {
-    if (this != &that) {
-      list_index = that.list_index;
-      tree_item = that.tree_item;
-    }
-    return *this;
+inline DisplayInfo *DboxMain::GetEntryGUIInfo(const CItemData &ci,
+                                              bool bAllowFail)
+{
+  auto E2G_iter = m_MapEntryToGUI.find(ci.GetUUID());
+  if (E2G_iter != m_MapEntryToGUI.end()) {
+    return &E2G_iter->second;
   }
-};
+
+  if (!bAllowFail) {
+    ASSERT(0); // caller expected the find to succeed
+  }
+  return nullptr;  
+}
