@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -45,9 +45,9 @@ struct CreateAccelTable {
   {
     if (p.second.siVirtKey != 0 && p.second.siVirtKey != m_ucAutotypeKey) {
       m_pacceltbl->fVirt = FVIRTKEY |
-               ((p.second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL ? FCONTROL : 0) |
-               ((p.second.cModifier & HOTKEYF_ALT)     == HOTKEYF_ALT     ? FALT     : 0) |
-               ((p.second.cModifier & HOTKEYF_SHIFT)   == HOTKEYF_SHIFT   ? FSHIFT   : 0);
+               ((p.second.cPWSModifier & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL ? FCONTROL : 0) |
+               ((p.second.cPWSModifier & PWS_HOTKEYF_ALT)     == PWS_HOTKEYF_ALT     ? FALT     : 0) |
+               ((p.second.cPWSModifier & PWS_HOTKEYF_SHIFT)   == PWS_HOTKEYF_SHIFT   ? FSHIFT   : 0);
       m_pacceltbl->key = (WORD)p.second.siVirtKey;
       m_pacceltbl->cmd = (WORD)p.first;
       m_pacceltbl++;
@@ -90,7 +90,6 @@ void InsertShortcuts(CMenu *pMenu, MapMenuShortcuts &mms,
   // if parentID == 0, we're processing a toplevel menu,
   // else, we're passed the menu in which the desired
   // submenu resides.
-  BOOL brc;
   wchar_t tcMenuString[_MAX_PATH + 1];
   CMenuShortcut mst;
   MENUITEMINFO miteminfo;
@@ -119,8 +118,7 @@ void InsertShortcuts(CMenu *pMenu, MapMenuShortcuts &mms,
     miteminfo.cch = _MAX_PATH;
     miteminfo.dwTypeData = tcMenuString;
 
-    brc = pSCTMenu->GetMenuItemInfo(ui, &miteminfo, TRUE);
-    ASSERT(brc != 0);
+    VERIFY(pSCTMenu->GetMenuItemInfo(ui, &miteminfo, TRUE));
 
     if (miteminfo.wID >= 1) {
       std::pair< MapMenuShortcutsIter, bool > pr;
@@ -140,14 +138,14 @@ void DboxMain::SetUpInitialMenuStrings()
   UINT uiCount;
 
   CString sKeyName;
-  BOOL brc;
 
   // Following are excluded from list of user-configurable
   // shortcuts
   UINT excludedMenuItems[] = {
   // Add user Excluded Menu Items - anything that is a Popup Menu
     ID_FILEMENU, ID_EXPORTMENU, ID_IMPORTMENU, ID_EDITMENU,
-    ID_VIEWMENU, ID_FILTERMENU, ID_CHANGEFONTMENU, ID_REPORTSMENU,
+    ID_VIEWMENU, ID_SUBVIEWMENU, ID_FILTERMENU,
+    ID_CHANGEFONTMENU, ID_REPORTSMENU,
     ID_MANAGEMENU, ID_LANGUAGEMENU, ID_HELPMENU, ID_FINDMENU,
     ID_EXPORTENTMENU, ID_EXPORTGROUPMENU,
 
@@ -163,8 +161,8 @@ void DboxMain::SetUpInitialMenuStrings()
   // for the action.  Add here to stop them being in the Shortcut
   // Options CListCtrl.
   // User can alter the function using the base values
-  // ID_MENUITEM_EDIT, ID_MENUITEM_DELETE & ID_MENUITEM_RENAME
-    ID_MENUITEM_VIEW,
+  // ID_MENUITEM_EDITENTRY, ID_MENUITEM_DELETE & ID_MENUITEM_RENAME
+    ID_MENUITEM_VIEWENTRY,
     ID_MENUITEM_DELETEENTRY, ID_MENUITEM_DELETEGROUP,
     ID_MENUITEM_RENAMEENTRY, ID_MENUITEM_RENAMEGROUP,
   };
@@ -174,8 +172,7 @@ void DboxMain::SetUpInitialMenuStrings()
                              excludedMenuItems + _countof(excludedMenuItems));
 
   pMainMenu = new CMenu;
-  brc = pMainMenu->LoadMenu(IDR_MAINMENU);
-  ASSERT(brc != 0);
+  VERIFY(pMainMenu->LoadMenu(IDR_MAINMENU));
 
   wchar_t tcMenuString[_MAX_PATH + 1];
 
@@ -190,7 +187,7 @@ void DboxMain::SetUpInitialMenuStrings()
 
   st_MenuShortcut st_mst;
   st_mst.siVirtKey;
-  st_mst.cModifier = HOTKEYF_ALT;
+  st_mst.cPWSModifier = PWS_HOTKEYF_ALT;
 
   for (UINT ui = 0; ui < uiCount; ui++) {
     SecureZeroMemory(tcMenuString, sizeof(tcMenuString));
@@ -201,8 +198,7 @@ void DboxMain::SetUpInitialMenuStrings()
     miteminfo.dwTypeData = tcMenuString;
     miteminfo.cch = _MAX_PATH;
 
-    brc = pMainMenu->GetMenuItemInfo(ui, &miteminfo, TRUE);
-    ASSERT(brc != 0);
+    VERIFY(pMainMenu->GetMenuItemInfo(ui, &miteminfo, TRUE));
 
     if (miteminfo.wID >= 1) {
       CString csMainMenuItem = tcMenuString;
@@ -218,17 +214,17 @@ void DboxMain::SetUpInitialMenuStrings()
   // Add 3 special keys F1 (for Help), Ctrl+Q/Alt+F4 (Exit)
   st_mst.nControlID = ID_MENUITEM_HELP;
   st_mst.siVirtKey = VK_F1;
-  st_mst.cModifier = 0;
+  st_mst.cPWSModifier = 0;
   m_ReservedShortcuts.push_back(st_mst);
 
   st_mst.nControlID = ID_MENUITEM_EXIT;
   st_mst.siVirtKey = 'Q';
-  st_mst.cModifier = HOTKEYF_CONTROL;
+  st_mst.cPWSModifier = PWS_HOTKEYF_CONTROL;
   m_ReservedShortcuts.push_back(st_mst);
 
   st_mst.nControlID = ID_MENUITEM_EXIT;
   st_mst.siVirtKey = VK_F4;
-  st_mst.cModifier = HOTKEYF_ALT;
+  st_mst.cPWSModifier = PWS_HOTKEYF_ALT;
   m_ReservedShortcuts.push_back(st_mst);
 
   // Now get all other Menu items
@@ -268,6 +264,9 @@ void DboxMain::SetUpInitialMenuStrings()
   ASSERT(isubmenu_pos != -1);
   pSubMenu = pMainMenu->GetSubMenu(isubmenu_pos);
 
+  // Do View Menu Subview submenu
+  InsertShortcuts(pSubMenu, m_MapMenuShortcuts, ID_SUBVIEWMENU);
+
   // Do View Menu Filter submenu
   InsertShortcuts(pSubMenu, m_MapMenuShortcuts, ID_FILTERMENU);
 
@@ -286,12 +285,10 @@ void DboxMain::SetUpInitialMenuStrings()
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_HELPMENU);
 
   // Don't need main menu again here
-  brc = pMainMenu->DestroyMenu();
-  ASSERT(brc != 0);
+  VERIFY(pMainMenu->DestroyMenu());
 
   // Do Find toolbar menu items not on a menu!
-  brc = pMainMenu->LoadMenu(IDR_POPFIND);
-  ASSERT(brc != 0);
+  VERIFY(pMainMenu->LoadMenu(IDR_POPFIND));
 
   // Again a parent menu (uiParentID == 0)
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, 0);
@@ -299,14 +296,13 @@ void DboxMain::SetUpInitialMenuStrings()
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_FINDMENU);
 
   // No longer need any menus
-  brc = pMainMenu->DestroyMenu();
-  ASSERT(brc != 0);
+  VERIFY(pMainMenu->DestroyMenu());
 
   delete pMainMenu;
 
   // Now that we have all menu strings - go get current accelerator strings
   // and update Map
-  MapMenuShortcutsIter iter, iter_entry, inuse_iter;
+  MapMenuShortcutsIter iter, iter_entry, iter_parent, inuse_iter;
   HACCEL curacctbl = app.m_ghAccelTable;
   //if (curacctbl != NULL) {
   //  DestroyAcceleratorTable(app.m_ghAccelTable);
@@ -339,11 +335,11 @@ void DboxMain::SetUpInitialMenuStrings()
       if (iter != m_MapMenuShortcuts.end()) {
         iter->second.siDefVirtKey = iter->second.siVirtKey = 
           (unsigned char)paccel->key;
-          iter->second.cDefModifier = iter->second.cModifier =
-          ((paccel->fVirt & FCONTROL) == FCONTROL ? HOTKEYF_CONTROL : 0) |
-          ((paccel->fVirt & FALT)     == FALT     ? HOTKEYF_ALT     : 0) |
-          ((paccel->fVirt & FSHIFT)   == FSHIFT   ? HOTKEYF_SHIFT   : 0) |
-          (IsExtended((int)paccel->key)           ? HOTKEYF_EXT     : 0);
+          iter->second.cDefPWSModifier = iter->second.cPWSModifier =
+          ((paccel->fVirt & FCONTROL) == FCONTROL ? PWS_HOTKEYF_CONTROL : 0) |
+          ((paccel->fVirt & FALT)     == FALT     ? PWS_HOTKEYF_ALT     : 0) |
+          ((paccel->fVirt & FSHIFT)   == FSHIFT   ? PWS_HOTKEYF_SHIFT   : 0) |
+          (IsExtended((int)paccel->key)           ? PWS_HOTKEYF_EXT     : 0);
       }
       paccel++;
     }
@@ -357,20 +353,12 @@ void DboxMain::SetUpInitialMenuStrings()
   // change shortcuts as per preferences
   std::vector<st_prefShortcut> vShortcuts(PWSprefs::GetInstance()->GetPrefShortcuts());
 
-  // We need to convert from PWS to Hotkey modifiers
-  for (size_t i = 0; i < vShortcuts.size(); i++) {
-    WORD wPWSModifiers = vShortcuts[i].cModifier;
-    // Translate from CHotKeyCtrl to PWS modifiers
-    WORD wHKModifiers = ConvertModifersPWS2MFC(wPWSModifiers);
-    vShortcuts[i].cModifier = (unsigned char)wHKModifiers;
-  }
-      
-  size_t N = vShortcuts.size();
+  const size_t N = vShortcuts.size();
   for (size_t i = 0; i < N; i++) {
     const st_prefShortcut &stxst = vShortcuts[i];
     // User should not have these sub-entries in their config file
     if (stxst.id == ID_MENUITEM_GROUPENTER  ||
-        stxst.id == ID_MENUITEM_VIEW        ||
+        stxst.id == ID_MENUITEM_VIEWENTRY   ||
         stxst.id == ID_MENUITEM_DELETEENTRY ||
         stxst.id == ID_MENUITEM_DELETEGROUP ||
         stxst.id == ID_MENUITEM_RENAME      ||
@@ -378,15 +366,29 @@ void DboxMain::SetUpInitialMenuStrings()
         stxst.id == ID_MENUITEM_RENAMEGROUP) {
       continue;
     }
+
     iter = m_MapMenuShortcuts.find(stxst.id);
     if (iter == m_MapMenuShortcuts.end()) {
       // Unknown Control ID - ignore - maybe used by a later version of PWS
       continue;
     }
+
+    // Update Menu name in vector
+    iter_parent = m_MapMenuShortcuts.find(iter->second.uiParentID);
+    std::wstring name(L"");
+    do {
+      name = iter_parent->second.name + std::wstring(L"->") + name;
+      iter_parent = m_MapMenuShortcuts.find(iter_parent->second.uiParentID);
+    } while (iter_parent != m_MapMenuShortcuts.end());
+
+    name += iter->second.name;
+    Remove(name, L'&');
+    vShortcuts[i].Menu_Name = name;
+
     // Check not already in use (ignore if deleting current shortcut)
     if (stxst.siVirtKey != 0) {
       st_mst.siVirtKey = stxst.siVirtKey;
-      st_mst.cModifier = stxst.cModifier;
+      st_mst.cPWSModifier = stxst.cPWSModifier;
       already_inuse inuse(st_mst);
       inuse_iter = std::find_if(m_MapMenuShortcuts.begin(),
                                 m_MapMenuShortcuts.end(),
@@ -397,19 +399,23 @@ void DboxMain::SetUpInitialMenuStrings()
         continue;
       }
     }
+
     if ((iter->second.siVirtKey  != stxst.siVirtKey  ||
-         iter->second.cModifier != stxst.cModifier)) {
-      // User changed a added or shortcut
+         iter->second.cPWSModifier != stxst.cPWSModifier)) {
+      // User changed or added a shortcut
       iter->second.siVirtKey  = stxst.siVirtKey;
-      iter->second.cModifier = stxst.cModifier;
+      iter->second.cPWSModifier = stxst.cPWSModifier;
     }
   }
 
+  // Update Menu names for later XML comment
+  PWSprefs::GetInstance()->SetPrefShortcuts(vShortcuts);
+
   // Set up the shortcuts based on the main entry
   // for View, Delete and Rename
-  iter = m_MapMenuShortcuts.find(ID_MENUITEM_EDIT);
+  iter = m_MapMenuShortcuts.find(ID_MENUITEM_EDITENTRY);
   ASSERT(iter != m_MapMenuShortcuts.end());
-  iter_entry = m_MapMenuShortcuts.find(ID_MENUITEM_VIEW);
+  iter_entry = m_MapMenuShortcuts.find(ID_MENUITEM_VIEWENTRY);
   ASSERT(iter_entry != m_MapMenuShortcuts.end());
   iter_entry->second.SetKeyFlags(iter->second);
 
@@ -465,7 +471,6 @@ void DboxMain::SetUpMenuStrings(CMenu *pPopupMenu)
   // format, Popup menus can have IDs
   ASSERT_VALID(pPopupMenu);
 
-  BOOL brc;
   MENUITEMINFO miteminfo = {0};
 
   MapMenuShortcutsIter iter;
@@ -478,8 +483,7 @@ void DboxMain::SetUpMenuStrings(CMenu *pPopupMenu)
     miteminfo.cbSize = sizeof(miteminfo);
     miteminfo.fMask = MIIM_ID | MIIM_STATE;
 
-    brc = pPopupMenu->GetMenuItemInfo(ui, &miteminfo, TRUE);
-    ASSERT(brc != 0);
+    VERIFY(pPopupMenu->GetMenuItemInfo(ui, &miteminfo, TRUE));
 
     // Exit & Help never changed and their shortcuts are in the menu text
     if (miteminfo.wID >= 1 &&
@@ -586,7 +590,8 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
   }
 
   // We have done for all except the Edit and View menus
-  if (uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU && uiMenuID != ID_FILTERMENU)
+  if (uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU &&
+      uiMenuID != ID_FILTERMENU && uiMenuID != ID_SUBVIEWMENU)
     return;
 
   // If View menu selected (contains 'Flattened &List' menu item)
@@ -602,15 +607,9 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOWHIDE_DRAGBAR, MF_BYCOMMAND |
                               bDragBarState ? MF_CHECKED : MF_UNCHECKED);
 
-    pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOWHIDE_UNSAVED, MF_BYCOMMAND |
-                              m_bUnsavedDisplayed ? MF_CHECKED : MF_UNCHECKED);
-
-    pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOW_ALL_EXPIRY, MF_BYCOMMAND |
-                              m_bExpireDisplayed ? MF_CHECKED : MF_UNCHECKED);
-
     // Don't show filter menu if "internal" menu active
     pPopupMenu->EnableMenuItem(ID_FILTERMENU, MF_BYCOMMAND |
-             (m_bUnsavedDisplayed || m_bExpireDisplayed) ? MF_GRAYED : MF_ENABLED);
+             (m_bUnsavedDisplayed || m_bExpireDisplayed || m_bFindFilterDisplayed) ? MF_GRAYED : MF_ENABLED);
 
     pPopupMenu->CheckMenuRadioItem(ID_MENUITEM_NEW_TOOLBAR,
                                    ID_MENUITEM_OLD_TOOLBAR,
@@ -622,6 +621,18 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     goto exit;
   }  // View menu
 
+  if (uiMenuID == ID_SUBVIEWMENU) {
+    pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOWHIDE_UNSAVED, MF_BYCOMMAND |
+                              m_bUnsavedDisplayed ? MF_CHECKED : MF_UNCHECKED);
+
+    pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOW_ALL_EXPIRY, MF_BYCOMMAND |
+                              m_bExpireDisplayed ? MF_CHECKED : MF_UNCHECKED);
+
+    pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOW_FOUNDENTRIES, MF_BYCOMMAND |
+                              m_bFindFilterDisplayed ? MF_CHECKED : MF_UNCHECKED);
+    goto exit;
+  } // Subview
+
   if (uiMenuID == ID_FILTERMENU) {
     pPopupMenu->ModifyMenu(1, MF_BYPOSITION,
                            m_bFilterActive ? ID_MENUITEM_CLEARFILTER : ID_MENUITEM_APPLYFILTER,
@@ -691,6 +702,22 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
           pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
         }
       }
+      
+      // Add Find Next/Previous if find entries were found
+      if (m_FindToolBar.EntriesFound()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_FIND, tc_dummy);
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_FINDUP, tc_dummy);
+      } 
+
+      // Only add "Find..." if find filter not active
+      if (!(m_bFilterActive && m_bFindFilterDisplayed)) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                                ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
+
       if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_UNDO, tc_dummy);
@@ -698,8 +725,8 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                ID_MENUITEM_REDO, tc_dummy);
         pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       }
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
+
+      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING, ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       goto exit;
     }
 
@@ -711,19 +738,21 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                ID_MENUITEM_ADD, tc_dummy);
       }
 
-      // Add Find Next/Previous if find still active and entries were found
-      if (m_FindToolBar.IsVisible() && m_FindToolBar.EntriesFound()) {
+      // Add Find Next/Previous if find entries were found
+      if (m_FindToolBar.EntriesFound()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FIND, tc_dummy);
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FINDUP, tc_dummy);
       } 
 
-      // Always add "Find..."
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                              ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+      // Only add "Find..." if find filter not active
+      if (!(m_bFilterActive && m_bFindFilterDisplayed)) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                                ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
 
-      pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_GROUPENTER, tc_dummy);
       
@@ -764,7 +793,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
           ID_MENUITEM_EXPORTGRP2DB, GGstr);
         GGstr.LoadString(IDS_EXPORTGRPMENU);
-        pPopupMenu->AppendMenu(MF_POPUP, (UINT)GGsubMenu.Detach(), GGstr);
+        pPopupMenu->AppendMenu(MF_POPUP, (UINT_PTR)GGsubMenu.Detach(), GGstr);
       }
 
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
@@ -775,8 +804,8 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                ID_MENUITEM_REDO, tc_dummy);
         pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       }
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
+
+      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING, ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       goto exit;
     }
 
@@ -808,8 +837,10 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       }
 
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             m_core.IsReadOnly() ? ID_MENUITEM_VIEW : ID_MENUITEM_EDIT,
+                             ((m_core.IsReadOnly() || pci->IsProtected()) ?
+                              ID_MENUITEM_VIEWENTRY : ID_MENUITEM_EDITENTRY),
                              tc_dummy);
+
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_DELETEENTRY, tc_dummy);
@@ -820,17 +851,20 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         }
       }
 
-      // Only have Find Next/Previous if find still active and entries were found
-      if (m_FindToolBar.IsVisible() && m_FindToolBar.EntriesFound()) {
+      // Only have Find Next/Previous if find entries were found
+      if (m_FindToolBar.EntriesFound()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FIND, tc_dummy);
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FINDUP, tc_dummy);
       }
 
-      // Always add "Find..."
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+      // Only add "Find..." if find filter not active
+      if (!(m_bFilterActive && m_bFindFilterDisplayed)) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                                ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
 
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
@@ -840,9 +874,8 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
           pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                  ID_MENUITEM_ADDGROUP, tc_dummy);
         }
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       }
-
-      pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       
       if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
@@ -933,10 +966,22 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         case CItemData::ET_ALIAS:
         case CItemData::ET_SHORTCUT:
           // Allow going to/editing the appropriate base entry
-          pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                                 ID_MENUITEM_GOTOBASEENTRY, tc_dummy); 
-          pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                                 ID_MENUITEM_EDITBASEENTRY, tc_dummy);
+          if (m_bFilterActive) {
+            // If a filter is active, then might not be able to go to
+            // entry's base entry as not in Tree or List view
+            pws_os::CUUID uuidBase = pci->GetBaseUUID();
+            auto iter = m_MapEntryToGUI.find(uuidBase);
+            ASSERT(iter != m_MapEntryToGUI.end());
+            if (iter->second.list_index != -1) {
+              pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                ID_MENUITEM_GOTOBASEENTRY, tc_dummy);
+              pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                ID_MENUITEM_EDITBASEENTRY, tc_dummy);
+            } else {
+              pPopupMenu->RemoveMenu(ID_MENUITEM_GOTOBASEENTRY, MF_BYCOMMAND);
+              pPopupMenu->RemoveMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND);
+            }
+          }
          break;
         default:
           ASSERT(0);
@@ -960,7 +1005,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       EEsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
                            ID_MENUITEM_EXPORTENT2DB, EEstr);
       EEstr.LoadString(IDS_EXPORTENTMENU);
-      pPopupMenu->AppendMenu(MF_POPUP, (UINT)EEsubMenu.Detach(), EEstr);
+      pPopupMenu->AppendMenu(MF_POPUP, (UINT_PTR)EEsubMenu.Detach(), EEstr);
 
       if (pci->IsShortcut() ? pbci->HasAttRef() : pci->HasAttRef()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
@@ -986,7 +1031,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
   }  // Edit menu
 
 exit:
-    SetUpMenuStrings(pPopupMenu);
+  SetUpMenuStrings(pPopupMenu);
 }
 
 // Helps with MRU by allowing ON_UPDATE_COMMAND_UI
@@ -1000,13 +1045,11 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
   // (ID_EDITMENU).
   ASSERT_VALID(pPopupMenu);
 
-  BOOL brc;
   MENUINFO minfo = {0};
   minfo.cbSize = sizeof(MENUINFO);
   minfo.fMask = MIM_MENUDATA;
 
-  brc = pPopupMenu->GetMenuInfo(&minfo);
-  ASSERT(brc != 0);
+  VERIFY(pPopupMenu->GetMenuInfo(&minfo));
 
   // Need to update the main menu if the shortcuts have been changed or
   // if the Edit or View menu.  The Edit menu is completely rebuilt each time
@@ -1020,6 +1063,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
     case ID_IMPORTMENU:
     case ID_EDITMENU:
     case ID_VIEWMENU:
+    case ID_SUBVIEWMENU:
     case ID_FILTERMENU:
     case ID_EXPORTENTMENU:
     case ID_EXPORTGROUPMENU:
@@ -1046,6 +1090,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
     // disable the menu item.
     int iLangPos = app.FindMenuItem(pPopupMenu, ID_LANGUAGEMENU);
     if (iLangPos >= 0) {
+      BOOL brc;
       CMenu *pSubMenu = pPopupMenu->GetSubMenu(iLangPos);
       brc = ProcessLanguageMenu(pSubMenu);
       pPopupMenu->EnableMenuItem(iLangPos, MF_BYPOSITION | (brc ? MF_ENABLED : MF_GRAYED));
@@ -1134,8 +1179,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
   minfo.fMask = MIM_STYLE;
   minfo.dwStyle = MNS_CHECKORBMP | MNS_AUTODISMISS;
 
-  brc = pPopupMenu->SetMenuInfo(&minfo);
-  ASSERT(brc != 0);
+  VERIFY(pPopupMenu->SetMenuInfo(&minfo));
 
   MENUITEMINFO miteminfo = {0};
   CRUEItemData *pmd;
@@ -1148,8 +1192,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
     miteminfo.cbSize = sizeof(miteminfo);
     miteminfo.fMask = MIIM_FTYPE | MIIM_DATA;
 
-    brc = pPopupMenu->GetMenuItemInfo(pos, &miteminfo, TRUE);
-    ASSERT(brc != 0);
+    VERIFY(pPopupMenu->GetMenuItemInfo(pos, &miteminfo, TRUE));
 
     pmd = (CRUEItemData *)miteminfo.dwItemData;
     if (pmd && pmd->IsRUEID() && !(miteminfo.fType & MFT_OWNERDRAW) &&
@@ -1160,8 +1203,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
       miteminfo.hbmpItem = HBMMENU_CALLBACK;
       miteminfo.fType = MFT_STRING;
 
-      brc = pPopupMenu->SetMenuItemInfo(pos, &miteminfo, TRUE);
-      ASSERT(brc != 0);
+      VERIFY(pPopupMenu->SetMenuItemInfo(pos, &miteminfo, TRUE));
     }
   }
 }
@@ -1171,7 +1213,6 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
 {
   const DWORD dwTrackPopupFlags = TPM_LEFTALIGN | TPM_RIGHTBUTTON;
 
-  BOOL brc;
   CPoint client;
   int item = -1;
   const CItemData *pci(NULL), *pbci(NULL);
@@ -1223,8 +1264,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       mp.y > rect.top && mp.y < rect.bottom) {
     if (menu.LoadMenu(IDR_POPCUSTOMIZETOOLBAR)) {
       minfo.dwMenuData = IDR_POPCUSTOMIZETOOLBAR;
-      brc = menu.SetMenuInfo(&minfo);
-      ASSERT(brc != 0);
+      VERIFY(menu.SetMenuInfo(&minfo));
 
       CMenu *pPopup = menu.GetSubMenu(0);
       ASSERT_VALID(pPopup);
@@ -1280,8 +1320,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       UpdateToolBarForSelectedItem(pci);
       if (pci != NULL) {
         // right-click was on an item (LEAF of some kind: normal, alias, shortcut)
-        DisplayInfo *pdi = (DisplayInfo *)pci->GetDisplayInfo();
-        ASSERT(pdi != NULL);
+        DisplayInfo *pdi = GetEntryGUIInfo(*pci);
         ASSERT(pdi->tree_item == ti);
         item = pdi->list_index;
         m_ctlItemTree.SelectItem(ti); // So that OnEdit gets the right one
@@ -1325,9 +1364,8 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       // not over anything
       if (menu.LoadMenu(IDR_POPTREE)) {  // "Add Group"
         minfo.dwMenuData = IDR_POPTREE;
-        brc = menu.SetMenuInfo(&minfo);
-        ASSERT(brc != 0);
-        CMenu* pPopup = menu.GetSubMenu(0);
+        VERIFY(menu.SetMenuInfo(&minfo));
+        CMenu *pPopup = menu.GetSubMenu(0);
         ASSERT_VALID(pPopup);
         
         ti = m_ctlItemTree.GetSelectedItem();
@@ -1352,12 +1390,10 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
 
     // Only compare entries if 2 entries are selected - List view only
     if (m_IsListView && m_ctlItemList.GetSelectedCount() == 2) {
-      brc = menu.LoadMenu(IDR_POPCOMPAREENTRIES);
-      ASSERT(brc != 0);
+      VERIFY(menu.LoadMenu(IDR_POPCOMPAREENTRIES));
 
       minfo.dwMenuData = IDR_POPCOMPAREENTRIES;
-      brc = menu.SetMenuInfo(&minfo);
-      ASSERT(brc != 0);
+      VERIFY(menu.SetMenuInfo(&minfo));
 
       pPopup = menu.GetSubMenu(0);
       ASSERT_VALID(pPopup);
@@ -1368,36 +1404,170 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
     }
 
     ASSERT(pci != NULL);
-    brc = menu.LoadMenu(IDR_POPEDITMENU);
-    ASSERT(brc != 0);
+    VERIFY(menu.LoadMenu(IDR_POPEDITMENU));
 
     minfo.dwMenuData = IDR_POPEDITMENU;
-    brc = menu.SetMenuInfo(&minfo);
-    ASSERT(brc != 0);
+    VERIFY(menu.SetMenuInfo(&minfo));
 
     pPopup = menu.GetSubMenu(0);
     ASSERT_VALID(pPopup);
 
+    if (m_core.IsReadOnly() || pci->IsProtected()) {
+      CString csMenu(MAKEINTRESOURCE(ID_MENUITEM_VIEWENTRY));
+      pPopup->ModifyMenu(ID_MENUITEM_EDITENTRY, MF_BYCOMMAND, ID_MENUITEM_VIEWENTRY, csMenu);
+    } else {
+      CString csMenu(MAKEINTRESOURCE(ID_MENUITEM_EDITENTRY));
+      pPopup->ModifyMenu(ID_MENUITEM_EDITENTRY, MF_BYCOMMAND, ID_MENUITEM_EDITENTRY, csMenu);
+    }
+
     const CItemData::EntryType etype_original = pci->GetEntryType();
+
+    if (pci->IsDependent()) {
+      pbci = m_core.GetBaseEntry(pci);
+    }
+
+    // Get list of UUIDs of a base's dependants.
+    // Add entry's [g:t:u] and its display position into a map, which will
+    // sort the entries.
+    UUIDVector tlist;
+    std::map<StringX, int> mapDependants;
+    int iMaxVisibleEntries(0);
+
+    if (etype_original == CItemData::ET_SHORTCUTBASE || etype_original == CItemData::ET_ALIASBASE) {
+      m_core.GetAllDependentEntries(pci->GetUUID(), tlist,
+        etype_original == CItemData::ET_SHORTCUTBASE ? CItemData::ET_SHORTCUT : CItemData::ET_ALIAS);
+
+      for (size_t i = 0; i < tlist.size(); i++) {
+        ItemListIter iter = Find(tlist[i]);
+        DisplayInfo *pdi = GetEntryGUIInfo(iter->second, true);
+
+        // Check in Tree (filter may be active and so may not be there)
+        int index = pdi->list_index;
+        if (pdi == NULL) {
+          index = -1;
+        } else {
+          iMaxVisibleEntries++;
+        }
+
+        StringX sxGroup = iter->second.GetGroup();
+        StringX sxTitle = iter->second.GetTitle();
+        StringX sxUser = iter->second.GetUser();
+        StringX sxEntry;
+        Format(sxEntry, L"\xab%s\xbb \xab%s\xbb \xab%s\xbb", sxGroup.c_str(),
+          sxTitle.c_str(), sxUser.c_str());
+
+        // Add to map which will sort by entry name
+        mapDependants.insert(make_pair(sxEntry, pdi->list_index));
+      }
+    }
+
+    // Set up "Go to Aliases/Shortcuts"
+    int nID = ID_MENUITEM_GOTODEPENDANT1;
+    m_vGotoDependants.clear();
+
+    // Find popup menu for this - not can't have a command ID for context menu
+    // popups and so must go by position. This is after "Edit Base Entry"
+    int isubmenu_pos = app.FindMenuItem(pPopup, ID_MENUITEM_EDITBASEENTRY) + 1;
+
+    // Get pointer to this popup
+    CMenu *pGotoDpdPopup = pPopup->GetSubMenu(isubmenu_pos);
+
     switch (etype_original) {
-      case CItemData::ET_NORMAL:
       case CItemData::ET_SHORTCUTBASE:
+      {
+        // Need to change menu text
+        CString csMenuText1(MAKEINTRESOURCE(IDS_GOTOSHORTCUTS));
+        pPopup->ModifyMenu(isubmenu_pos, MF_BYPOSITION, 0, csMenuText1);
+
+        // Remove dummy separator in menu definition
+        pGotoDpdPopup->RemoveMenu(0, MF_BYPOSITION);
+
+        // Got through shortcuts and add to popup menu
+        // Fill the dependant vector with its corresponding position to be used
+        // by the command to select it if the user selects it
+        for (auto iter = mapDependants.begin(); iter != mapDependants.end(); iter++) {
+          StringX sxEntry = iter->first;
+
+          if (iter->second == -1 || nID > ID_MENUITEM_GOTODEPENDANTMAX) {
+            // Dependant is NOT available or reached maximum
+            pGotoDpdPopup->InsertMenu((UINT)-1, MF_BYPOSITION | MF_STRING | MF_GRAYED,
+                                      0, sxEntry.c_str());
+          } else {
+            // Dependant is visible
+            m_vGotoDependants.push_back(iter->second);
+            pGotoDpdPopup->InsertMenu((UINT)-1, MF_BYPOSITION | MF_STRING, nID, sxEntry.c_str());
+            nID++;
+          }
+        }
+
+        // As a base entry - remove menu items for a base
+        pPopup->RemoveMenu(ID_MENUITEM_GOTOBASEENTRY, MF_BYCOMMAND);
+        pPopup->RemoveMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND);
+        break;
+      }
+      case CItemData::ET_NORMAL:
+        // As a normal entry - remove menu items for a base
+        pPopup->RemoveMenu(isubmenu_pos, MF_BYPOSITION);
         pPopup->RemoveMenu(ID_MENUITEM_GOTOBASEENTRY, MF_BYCOMMAND);
         pPopup->RemoveMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND);
         break;
       case CItemData::ET_ALIASBASE:
+        // Remove dummy separator in menu definition
+        pGotoDpdPopup->RemoveMenu(0, MF_BYPOSITION);
+
+        // Got throogh aliases and add to popup menu
+        // Fill the dependant vector with its corresponding position to be used
+        // by the command to select it if the user selects it
+        for (auto iter = mapDependants.begin(); iter != mapDependants.end(); iter++) {
+          StringX sxEntry = iter->first;
+          if (iter->second == -1 || nID > ID_MENUITEM_GOTODEPENDANTMAX) {
+            // Dependant is NOT available or reached maximum
+            pGotoDpdPopup->InsertMenu((UINT)-1, MF_BYPOSITION | MF_STRING | MF_GRAYED,
+              0, sxEntry.c_str());
+          } else {
+            // Dependant is visible
+            m_vGotoDependants.push_back(iter->second);
+            pGotoDpdPopup->InsertMenu((UINT)-1, MF_BYPOSITION | MF_STRING, nID, sxEntry.c_str());
+            nID++;
+          }
+        }
+
+        // As a base entry - remove menu items for a base - also can't have a shoretcut to an alias
         pPopup->RemoveMenu(ID_MENUITEM_CREATESHORTCUT, MF_BYCOMMAND);
         pPopup->RemoveMenu(ID_MENUITEM_GOTOBASEENTRY, MF_BYCOMMAND);
         pPopup->RemoveMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND);
         break;
       case CItemData::ET_ALIAS:
       case CItemData::ET_SHORTCUT:
+        pPopup->RemoveMenu(isubmenu_pos, MF_BYPOSITION);
         pPopup->RemoveMenu(ID_MENUITEM_CREATESHORTCUT, MF_BYCOMMAND);
+        if (m_bFilterActive) {
+          // If a filter is active, then might not be able to go to
+          // entry's base entry as not in Tree or List view
+          pws_os::CUUID uuidBase = pci->GetBaseUUID();
+          auto iter = m_MapEntryToGUI.find(uuidBase);
+          ASSERT(iter != m_MapEntryToGUI.end());
+          if (iter->second.list_index == -1) {
+            pPopup->RemoveMenu(ID_MENUITEM_GOTOBASEENTRY, MF_BYCOMMAND);
+            pPopup->RemoveMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND);
+          }
+        } else {
+          if (m_core.IsReadOnly() || pbci->IsProtected()) {
+            CString csMenuText(MAKEINTRESOURCE(IDS_VIEWBASEENTRY));
+            pPopup->ModifyMenu(ID_MENUITEM_EDITBASEENTRY, MF_BYCOMMAND, 
+                               ID_MENUITEM_EDITBASEENTRY, csMenuText);
+          }
+        }
         break;
       default:
         ASSERT(0);
     }
 
+    // Clear up list of dependant UUIDs and map between the dependant [g:t:u] and
+    // display position in list
+    tlist.clear();
+    mapDependants.clear();
+
     if (m_core.IsReadOnly() || pci->IsShortcut()) {
       pPopup->RemoveMenu(ID_MENUITEM_PROTECT, MF_BYCOMMAND);
       pPopup->RemoveMenu(ID_MENUITEM_UNPROTECT, MF_BYCOMMAND);
@@ -1405,10 +1575,6 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       pPopup->RemoveMenu(pci->IsProtected() ? ID_MENUITEM_PROTECT : ID_MENUITEM_UNPROTECT, MF_BYCOMMAND);
     }
 
-    if (pci->IsDependent()) {
-      pbci = m_core.GetBaseEntry(pci);
-    }
-
     bool bCopyEmail = !pci->IsFieldValueEmpty(CItemData::EMAIL, pbci);
     bool bSendEmail = bCopyEmail ||
                (!pci->IsFieldValueEmpty(CItemData::URL, pbci) && pci->IsURLEmail(pbci));
@@ -1476,11 +1642,11 @@ void DboxMain::SetupSpecialShortcuts()
     iter_group = m_MapMenuShortcuts.find(ID_MENUITEM_DELETEGROUP);
     iter_group->second.SetKeyFlags(iter->second);
 
-    m_wpDeleteMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+    m_wpDeleteMsg = ((iter->second.cPWSModifier & PWS_HOTKEYF_ALT) == PWS_HOTKEYF_ALT) ?
                            WM_SYSKEYDOWN : WM_KEYDOWN;
     m_wpDeleteKey = iter->second.siVirtKey;
-    m_bDeleteCtrl = (iter->second.cModifier  & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
-    m_bDeleteShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
+    m_bDeleteCtrl = (iter->second.cPWSModifier  & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL;
+    m_bDeleteShift = (iter->second.cPWSModifier & PWS_HOTKEYF_SHIFT) == PWS_HOTKEYF_SHIFT;
   } else {
     m_wpDeleteKey = 0;
   }
@@ -1495,11 +1661,11 @@ void DboxMain::SetupSpecialShortcuts()
     iter_group = m_MapMenuShortcuts.find(ID_MENUITEM_RENAMEGROUP);
     iter_group->second.SetKeyFlags(iter->second);
 
-    m_wpRenameMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+    m_wpRenameMsg = ((iter->second.cPWSModifier & PWS_HOTKEYF_ALT) == PWS_HOTKEYF_ALT) ?
                            WM_SYSKEYDOWN : WM_KEYDOWN;
     m_wpRenameKey = iter->second.siVirtKey;
-    m_bRenameCtrl = (iter->second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
-    m_bRenameShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
+    m_bRenameCtrl = (iter->second.cPWSModifier & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL;
+    m_bRenameShift = (iter->second.cPWSModifier & PWS_HOTKEYF_SHIFT) == PWS_HOTKEYF_SHIFT;
   } else {
     m_wpRenameKey = 0;
   }
@@ -1509,13 +1675,13 @@ void DboxMain::SetupSpecialShortcuts()
   
   // Save for CTreeCtrl & CListCtrl PreTranslateMessage
   if (iter != m_MapMenuShortcuts.end()) {
-    m_wpAutotypeDNMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+    m_wpAutotypeDNMsg = ((iter->second.cPWSModifier & PWS_HOTKEYF_ALT) == PWS_HOTKEYF_ALT) ?
                                WM_SYSKEYDOWN : WM_KEYDOWN;
-    m_wpAutotypeUPMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+    m_wpAutotypeUPMsg = ((iter->second.cPWSModifier & PWS_HOTKEYF_ALT) == PWS_HOTKEYF_ALT) ?
                                WM_SYSKEYUP : WM_KEYUP;
     m_wpAutotypeKey = iter->second.siVirtKey;
-    m_bAutotypeCtrl = (iter->second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
-    m_bAutotypeShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
+    m_bAutotypeCtrl = (iter->second.cPWSModifier & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL;
+    m_bAutotypeShift = (iter->second.cPWSModifier & PWS_HOTKEYF_SHIFT) == PWS_HOTKEYF_SHIFT;
   } else {
     m_wpAutotypeKey = 0;
   }
@@ -1554,7 +1720,7 @@ bool DboxMain::ProcessLanguageMenu(CMenu *pPopupMenu)
 }
 
 const unsigned int DboxMain::GetMenuShortcut(const unsigned short int &siVirtKey,
-                                             const unsigned char &cModifier,
+                                             const unsigned char &cPWSModifier,
                                              StringX &sxMenuItemName)
 {
   unsigned int nControlID(0);
@@ -1563,7 +1729,7 @@ const unsigned int DboxMain::GetMenuShortcut(const unsigned short int &siVirtKey
 
   st_MenuShortcut st_mst;
   st_mst.siVirtKey = siVirtKey;
-  st_mst.cModifier = cModifier;
+  st_mst.cPWSModifier = cPWSModifier;
   
   inuse_iter = std::find_if(m_MapMenuShortcuts.begin(),
                             m_MapMenuShortcuts.end(),
