@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,13 +17,15 @@ public:
 
   CPWPropertySheet(LPCTSTR pszCaption, CWnd* pParent, const bool bLongPPs);
 
+  // Following override to stop accelerators interfering (can't be protected as normal)
+  INT_PTR DoModal();
+
+protected:
   // Following override to reset idle timeout on any event
   virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
   // Following override to stop accelerators interfering
-  virtual INT_PTR DoModal();
   virtual BOOL OnInitDialog();
 
-protected:
   DECLARE_DYNAMIC(CPWPropertySheet)
 
   DboxMain *GetMainDlg() const;
