@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -10,7 +10,6 @@
 
 #include "stdafx.h"
 
-#include "Windowsdefs.h"
 #include "PasswordSafe.h"
 #include "ThisMfcApp.h"
 #include "GeneralMsgBox.h"
@@ -41,7 +40,7 @@ static char THIS_FILE[] = __FILE__;
 CPasskeyChangeDlg::CPasskeyChangeDlg(CWnd* pParent)
   : CPKBaseDlg(CPasskeyChangeDlg::IDD, pParent),
     m_LastFocus(IDC_PASSKEY), m_Yubi1pressed(false), m_Yubi2pressed(false),
-    m_oldpasskeyConfirmed(false)
+    m_oldpasskeyConfirmed(false), m_btnShowCombination(FALSE)
 {
   m_newpasskey = L"";
   m_confirmnew = L"";
@@ -67,15 +66,19 @@ void CPasskeyChangeDlg::DoDataExchange(CDataExchange* pDX)
   DDX_Control(pDX, IDC_CONFIRMNEW, *m_pctlConfirmNew);
   DDX_Control(pDX, IDC_NEWPASSKEY, *m_pctlNewPasskey);
   DDX_Control(pDX, IDC_PASSKEY, *m_pctlPasskey);
+
+  DDX_Check(pDX, IDC_SHOWCOMBINATION, m_btnShowCombination);
 }
 
 BEGIN_MESSAGE_MAP(CPasskeyChangeDlg, CPKBaseDlg)
   ON_WM_TIMER()
 
-  ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_STN_CLICKED(IDC_VKB, OnVirtualKeyboard)
+
+  ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDC_YUBIKEY2_BTN, OnYubikey2Btn)
   ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
+  ON_BN_CLICKED(IDC_SHOWCOMBINATION, OnShowCombination)
 
   ON_EN_SETFOCUS(IDC_PASSKEY, OnPasskeySetfocus)
   ON_EN_SETFOCUS(IDC_NEWPASSKEY, OnNewPasskeySetfocus)
@@ -88,6 +91,7 @@ BOOL CPasskeyChangeDlg::OnInitDialog()
 {
   CPKBaseDlg::OnInitDialog();
 
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
   Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_NEWPASSKEY));
   Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_CONFIRMNEW));
 
@@ -108,7 +112,7 @@ BOOL CPasskeyChangeDlg::OnInitDialog()
     GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
   }
 
-  return TRUE;
+  return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 void CPasskeyChangeDlg::yubiInserted(void)
@@ -141,7 +145,7 @@ void CPasskeyChangeDlg::OnOK()
     gmb.AfxMessageBox(IDS_WRONGOLDPHRASE);
   else if (rc == PWScore::CANT_OPEN_FILE)
     gmb.AfxMessageBox(IDS_CANTVERIFY);
-  else if (m_confirmnew != m_newpasskey)
+  else if (m_btnShowCombination == FALSE && m_confirmnew != m_newpasskey)
     gmb.AfxMessageBox(IDS_NEWOLDDONOTMATCH);
   else if (m_newpasskey.IsEmpty())
     gmb.AfxMessageBox(IDS_CANNOTBEBLANK);
@@ -158,7 +162,7 @@ void CPasskeyChangeDlg::OnOK()
 #ifndef PWS_FORCE_STRONG_PASSPHRASE
     cs_text.LoadString(IDS_USEITANYWAY);
     cs_msg += cs_text;
-    rc = gmb.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_ICONSTOP);
+    rc = (int)gmb.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_ICONSTOP);
     if (rc == IDYES)
       CPKBaseDlg::OnOK();
 #else
@@ -196,6 +200,36 @@ void CPasskeyChangeDlg::OnConfirmNewSetfocus()
   m_LastFocus = IDC_CONFIRMNEW;
 }
 
+void CPasskeyChangeDlg::OnShowCombination()
+{
+  UpdateData(TRUE);
+
+  m_pctlPasskey->SetSecure(m_btnShowCombination == TRUE ? FALSE : TRUE);
+  m_pctlNewPasskey->SetSecure(m_btnShowCombination == TRUE ? FALSE : TRUE);
+
+  if (m_btnShowCombination == TRUE) {
+    m_pctlPasskey->SetPasswordChar(0);
+    m_pctlPasskey->SetWindowText(m_passkey);
+
+    m_pctlNewPasskey->SetPasswordChar(0);
+    m_pctlNewPasskey->SetWindowText(m_newpasskey);
+
+    m_pctlConfirmNew->SetPasswordChar(0);
+    m_pctlConfirmNew->EnableWindow(FALSE);
+    m_pctlConfirmNew->SetWindowText(L"");
+  } else {
+    m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
+    m_pctlPasskey->SetSecureText(m_passkey);
+
+    m_pctlNewPasskey->SetPasswordChar(PSSWDCHAR);
+    m_pctlNewPasskey->SetSecureText(m_newpasskey);
+
+    m_pctlConfirmNew->SetPasswordChar(PSSWDCHAR);
+    m_pctlConfirmNew->EnableWindow(TRUE);
+    m_pctlConfirmNew->SetWindowText(L"");
+  }
+}
+
 void CPasskeyChangeDlg::OnVirtualKeyboard()
 {
   // Shouldn't be here if couldn't load DLL. Static control disabled/hidden
@@ -204,7 +238,7 @@ void CPasskeyChangeDlg::OnVirtualKeyboard()
 
   if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->IsWindowVisible()) {
     // Already there - move to top
-    m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    m_pVKeyBoardDlg->SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     return;
   }
 
@@ -218,7 +252,7 @@ void CPasskeyChangeDlg::OnVirtualKeyboard()
   }
 
   // Now show it and make it top
-  m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
+  m_pVKeyBoardDlg->SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
 
   return;
 }
@@ -281,6 +315,7 @@ void CPasskeyChangeDlg::OnYubikeyBtn()
 {
   // This is for existing password verification
   UpdateData(TRUE);
+
   m_Yubi1pressed = true;
   yubiRequestHMACSha1(m_passkey);
 }
@@ -288,7 +323,8 @@ void CPasskeyChangeDlg::OnYubikeyBtn()
 void CPasskeyChangeDlg::OnYubikey2Btn()
 {
   UpdateData(TRUE);
-  if (m_confirmnew != m_newpasskey) {
+
+  if (m_btnShowCombination == FALSE && m_confirmnew != m_newpasskey) {
     CGeneralMsgBox gmb;
     gmb.AfxMessageBox(IDS_NEWOLDDONOTMATCH);
   } else {
