@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,7 +14,6 @@
 #include "PKBaseDlg.h"
 #include "ControlExtns.h"
 
-class CVKeyBoardDlg;
 class DboxMain;
 class PWScore;
 
@@ -29,14 +28,20 @@ protected:
   // Dialog Data
   //{{AFX_DATA(CPasskeySetup)
   enum { IDD = IDD_PASSKEYSETUP };
-  CSecString m_verify;
   //}}AFX_DATA
 
+  CSecString m_verify;
+  BOOL m_btnShowCombination;
+
   virtual BOOL OnInitDialog();
   //{{AFX_VIRTUAL(CPasskeySetup)
-  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
+  virtual void DoDataExchange(CDataExchange *pDX);    // DDX/DDV support
   //}}AFX_VIRTUAL
 
+  void ProcessPhrase(); // Check the passphrase, call OnOK, OnCancel or just return
+  void YubiFailed(); // If YubiKey failed, offer to initialize it.
+  void YubiInitialize(); // called if YubiFailed and user confirmed
+
   // Implementation
   // Generated message map functions
   //{{AFX_MSG(CPasskeySetup)
@@ -47,13 +52,12 @@ protected:
   afx_msg void OnVerifykeySetfocus();
   afx_msg void OnVirtualKeyboard();
   afx_msg void OnYubikeyBtn();
-  void ProcessPhrase(); // Check the passphrase, call OnOK, OnCancel or just return
-  void YubiFailed(); // If YubiKey failed, offer to initialize it.
-  void YubiInitialize(); // called if YubiFailed and user confirmed
+  afx_msg void OnShowCombination();
   afx_msg LRESULT OnInsertBuffer(WPARAM, LPARAM);
   //}}AFX_MSG
 
   DECLARE_MESSAGE_MAP()
+
 private:
   DboxMain *m_pDbx;
   CSecEditExtn *m_pctlVerify;
