@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -8,8 +8,13 @@
 // RichEditCtrlExtns.cpp
 //
 
+// Used in AboutDlg and GeneralMsgBox ONLY where html links are required
+// NOT used in AddEdit_Basic for Notes - it uses CRichEditExtn defined in "ControlExtns.h"
+
 #include "stdafx.h"
+
 #include "RichEditCtrlExtn.h"
+
 #include <algorithm>
 #include <vector>
 #include <string>
@@ -184,6 +189,15 @@ void CRichEditCtrlExtn::SetWindowText(LPCWSTR lpszString)
 
   CString cs_formatstring(lpszString);
   CString cs_plaintext = GetTextFormatting(cs_formatstring, iError);
+
+  // Disable Font Binding in Rich Edit (http://msdn.microsoft.com/en-us/library/windows/desktop/hh298436%28v=vs.85%29.aspx)
+  // When it enabled, control tries to change font for chars, that it think aren't present in default font
+  // For example, for non-English messages, it can change font from MS Sans to Calibri when it found e.g.
+  // "french quotes" ('', ''). The worst in this case that it also may change font size and then not return
+  // all back, so the last part of the message will be printed using different font type/size
+  // Example of code taken from http://blogs.msdn.com/b/michkap/archive/2006/02/13/531110.aspx
+  CRichEditCtrl::SendMessage(EM_SETLANGOPTIONS, 0, (LPARAM)(CRichEditCtrl::SendMessage(EM_GETLANGOPTIONS, 0, 0) & ~IMF_AUTOFONT));
+
   if (iError != 0) {
     // Had an error - show unchanged text
     CRichEditCtrl::SetWindowText(cs_formatstring);
@@ -220,13 +234,8 @@ void CRichEditCtrlExtn::SetWindowText(LPCWSTR lpszString)
           cf2.yHeight = (format_iter->iSize) * 20;
         } else if (format_iter->entrytype == Name) {
           cf2.dwMask = CFM_FACE;
-#if (_MSC_VER >= 1400)
           memcpy_s(cf2.szFaceName, sizeof(cf2.szFaceName),
                    format_iter->tcszFACENAME, sizeof(format_iter->tcszFACENAME));
-#else
-          memcpy(cf2.szFaceName, Name_iter->tcszFACENAME, 
-                 sizeof(format_iter->tcszFACENAME));
-#endif
         }
         SetSelectionCharFormat(cf2);
       }
@@ -487,11 +496,7 @@ vnext:
           cur_format.iStart = iFontChangeStart;
           cur_format.iEnd = iTextPosition;
           SecureZeroMemory(cur_format.tcszFACENAME, sizeof(cur_format.tcszFACENAME));
-#if (_MSC_VER >= 1400)
           wcscpy_s(cur_format.tcszFACENAME, LF_FACESIZE, (LPCWSTR)csLastFaceName);
-#else
-          wcscpy(cur_format.tcszFACENAME, (LPCWSTR)csLastFaceName);
-#endif
           m_vFormat.push_back(cur_format);
           vLastFacenames.pop_back();
         }
@@ -558,6 +563,7 @@ vnext:
               }
 
               csFontVerb = csHTMLTag.Tokenize(L"\"", curFontPos);
+              csFontVerb.TrimLeft();
               if (csFontVerb.IsEmpty() && curFontPos == -1)
                 break;
               csFontVerbValue = csHTMLTag.Tokenize(L"\"", curFontPos);
@@ -579,11 +585,7 @@ vnext:
           csURL = csHTMLTag.Mid(8, dwTEnd - 8);
           if (!csURL.IsEmpty()) {
             csURL.MakeLower();
-#if (_MSC_VER >= 1400)
             wcscpy_s(this_ALink.tcszURL, _MAX_PATH, csURL);
-#else
-            wcscpy(this_ALink.tcszURL, csURL);
-#endif
           }
         }
         // Now get Friendly Name (note doing this within the while loop!)
@@ -628,11 +630,7 @@ COLORREF CRichEditCtrlExtn::ConvertColourToColorRef(CString &csValue)
     // Convert HTML to COLORREF
     ASSERT(csValue.GetLength() == 7);
     int icolour;
-#if (_MSC_VER >= 1400)
     swscanf_s(csValue.Mid(1), L"%06x", &icolour);
-#else
-    swscanf(csValue.Mid(1), L"%06x", &icolour);
-#endif
     int ired = (icolour & 0xff0000) >> 16;
     int igreen = (icolour & 0xff00);
     int iblue = (icolour & 0xff) << 16;
