@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -160,14 +160,13 @@
 //#include "./graphics/toolbar/classic/viewreports.xpm"
 //#include "./graphics/toolbar/classic/viewreports_disabled.xpm"
 
-
 enum {ID_SEPARATOR = -1};
 
 #define PWS_TOOLBAR_BITMAPS(n)  wxCONCAT(n, _xpm),                              \
                                 wxCONCAT(wxCONCAT(n, _disabled), _xpm),         \
                                 wxCONCAT(wxCONCAT(classic_, n), _xpm),          \
                                 wxCONCAT(wxCONCAT(classic_, n), _disabled_xpm)
-#define SEPARATOR {ID_SEPARATOR, wxT(""), NULL, NULL, NULL, NULL}
+#define SEPARATOR {ID_SEPARATOR, wxEmptyString, NULL, NULL, NULL, NULL}
 
 struct _PwsToolbarInfo{
   int id;
@@ -176,8 +175,8 @@ struct _PwsToolbarInfo{
   const char** bitmap_disabled;
   const char** bitmap_classic;
   const char** bitmap_classic_disabled;
-  // Following ctor's required to shut up some compier warnings
-_PwsToolbarInfo() : id(0), tooltip(wxT("")), bitmap_normal(NULL), bitmap_disabled(NULL),
+  // Following ctor's required to shut up some compiler warnings
+_PwsToolbarInfo() : id(0), tooltip(wxEmptyString), bitmap_normal(NULL), bitmap_disabled(NULL),
     bitmap_classic(NULL), bitmap_classic_disabled(NULL) {}
 _PwsToolbarInfo(int aid, const wxString &atooltip, const char** abitmap_normal,
                 const char** abitmap_disabled, const char** abitmap_classic, const char** abitmap_classic_disabled) :
