@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -139,10 +139,10 @@ bool AddEditPropSheet::Create( wxWindow* parent, wxWindowID id, const wxString&
   int flags = (m_type == VIEW) ? (wxCLOSE|wxHELP) : (wxOK|wxCANCEL|wxHELP);
   CreateButtons(flags);
   CreateControls();
-  LayoutDialog();
   Centre();
 ////@end AddEditPropSheet creation
   ItemFieldsToPropSheet();
+  LayoutDialog();
   return true;
 }
 
@@ -789,11 +789,6 @@ static struct {short pv; wxString name;}
   short defDCA =  short(PWSprefs::GetInstance()->
       GetPref(isShift ?
         PWSprefs::ShiftDoubleClickAction : PWSprefs::DoubleClickAction));
-  if (useDefault) {
-    iDCA = defDCA;
-  } else {
-    iDCA = dca;
-  }
   // Following loop:
   // - sets CB's client data to pref value
   // - Adds " (default)" to default string
@@ -804,8 +799,11 @@ static struct {short pv; wxString name;}
       wxString dv = dcaMapping[i].name;
       dv += wxT(" ("); dv += _("default"); dv += wxT(")");
       pcbox->SetString(i, dv);
+      if (useDefault || iDCA == defDCA) {
+        pcbox->SetValue(dv);
+      }
     }
-    if (iDCA == dcaMapping[i].pv)
+    else if (iDCA == dcaMapping[i].pv)
       pcbox->SetValue(dcaMapping[i].name);
   }
 }
@@ -813,7 +811,7 @@ static struct {short pv; wxString name;}
 void AddEditPropSheet::UpdateExpTimes()
 {
   // From m_item to display
-  
+
   m_item.GetXTime(m_tttXTime);
   m_item.GetXTimeInt(m_XTimeInt);
   m_XTime = m_CurXTime = m_item.GetXTimeL().c_str();
@@ -887,6 +885,21 @@ void AddEditPropSheet::ItemFieldsToPropSheet()
   m_url = m_item.GetURL().c_str();
   m_email = m_item.GetEmail().c_str();
   m_password = m_item.GetPassword();
+
+  if (m_item.IsAlias()) {
+    // Update password to alias form
+    // Show text stating that it is an alias
+
+    const CItemData *pbci = m_core.GetBaseEntry(&m_item);
+    ASSERT(pbci);
+    if (pbci) {
+      m_password = L"[" +
+                pbci->GetGroup() + L":" +
+                pbci->GetTitle() + L":" +
+                pbci->GetUser()  + L"]";
+    }
+  } // IsAlias
+
   PWSprefs *prefs = PWSprefs::GetInstance();
   if (prefs->GetPref(PWSprefs::ShowPWDefault)) {
     ShowPassword();
@@ -1108,6 +1121,8 @@ void AddEditPropSheet::HidePassword()
 static short GetSelectedDCA(const wxComboBox *pcbox,
                            short lastval, short defval)
 {
+  UNREFERENCED_PARAMETER(lastval);
+
   int sel = pcbox->GetSelection();
   intptr_t ival = -1;
   if (sel == wxNOT_FOUND) { // no selection
@@ -1233,7 +1248,26 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
                      m_symbols    != m_item.GetSymbols().c_str()     ||
                      oldPWP       != pwp);
 
-      bIsPSWDModified = (password != m_item.GetPassword());
+
+        if (!m_item.IsAlias()) {
+          bIsPSWDModified = (password != m_item.GetPassword());
+        }
+        else {
+          // Update password to alias form
+          // Show text stating that it is an alias
+          const CItemData *pbci = m_core.GetBaseEntry(&m_item);
+          ASSERT(pbci);
+          if (pbci) {
+            StringX alias = L"[" +
+                pbci->GetGroup() + L":" +
+                pbci->GetTitle() + L":" +
+                pbci->GetUser()  + L"]";
+            bIsPSWDModified = (password != alias);
+          }
+          else {
+            bIsPSWDModified = true;
+          }
+        }
 
       if (bIsModified) {
         // Just modify all - even though only 1 may have actually been modified
@@ -1259,6 +1293,18 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
         m_item.SetPolicyName(tostringx(polName));
         m_item.SetDCA(m_DCA);
         m_item.SetShiftDCA(m_ShiftDCA);
+        // Check for Group/Username/Title uniqueness
+        auto listindex = m_core.Find(m_item.GetGroup(), m_item.GetTitle(), m_item.GetUser());
+        if (listindex != m_core.GetEntryEndIter()) {
+          auto listItem = m_core.GetEntry(listindex);
+          if (listItem.GetUUID() != m_item.GetUUID()) {
+            wxMessageDialog msg(this,
+                                _("An entry or shortcut with the same Group, Title and Username already exists."),
+                                _("Error"), wxOK|wxICON_ERROR);
+            msg.ShowModal();
+            return;
+          }
+        }
       } // bIsModified
 
       time(&t);
@@ -1295,6 +1341,15 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
       m_item.SetUser(m_user.empty() ?
                      PWSprefs::GetInstance()->
                       GetPref(PWSprefs::DefaultUsername).c_str() : m_user.c_str());
+      // Check for Group/Username/Title uniqueness
+      if (m_core.Find(m_item.GetGroup(), m_item.GetTitle(), m_item.GetUser()) !=
+          m_core.GetEntryEndIter()) {
+        wxMessageDialog msg(this,
+                            _("An entry or shortcut with the same Group, Title and Username already exists."),
+                            _("Error"), wxOK|wxICON_ERROR);
+        msg.ShowModal();
+        return;
+      }
       m_item.SetNotes(tostringx(m_notes));
       m_item.SetURL(tostringx(m_url));
       m_item.SetEmail(tostringx(m_email));
